PRO LDRS_ESTIMATE_ROTATION_FROM_REFSTARS, calFT, fileNr, imgNr, tag, r, sx, sy, tx, ty
PRINT, 'LDRS_ESTIMATE_ROTATION_FROM_REFSTARS, .., ', fileNr, ', ', imgNr, ', ...'
IF NOT OBJ_VALID(calFT) THEN BEGIN
    PRINT, '  calFT is not an object => exiting'
    RETURN
ENDIF
file = calFT->GetFile(fileNr)
IF NOT OBJ_VALID(file) THEN BEGIN
    PRINT, '  file ', fileNr, ' not found => exiting'
    RETURN
ENDIF
nImages = file->GetNImages()
; 1. find the configuration of the image and store it in icfg
icfg = file->GetImageCFG(imgNr)
IF NOT OBJ_VALID(icfg) THEN BEGIN
    PRINT, '  cfg for image ', imgNr, ' not found => exiting'
    RETURN
ENDIF
isRef = icfg->GetBoolean('ISREF', DEFAULT=0)
IF isRef THEN BEGIN
    PRINT, '  image is the reference image => nothing changed'
    RETURN ; do not change the values for the reference image
ENDIF
; 2. find the reference image and store the configuration in rcfg
rcfg = OBJ_NEW()
refTX = 0.0
refTY = 0.0
FOR nr=0,nImages-1 DO BEGIN
    cfg = file->GetImageCFG(nr)
    IF NOT OBJ_VALID(cfg) THEN CONTINUE
    flag = cfg->GetBoolean('ISREF', DEFAULT=0)
    If flag THEN BEGIN
        rcfg = cfg
        hcfg = cfg->LocateByName(tag, /NO_CREATE)
        IF OBJ_VALID(hcfg) THEN BEGIN
            refRot = hcfg->GetFloat('ROTATE', DEFAULT=0.0)
            hcfg->GetFloat2D, 'TRANSLATE', refTX, refTY
        ENDIF
        BREAK
    ENDIF
ENDFOR
IF NOT OBJ_VALID(rcfg) THEN BEGIN
    PRINT, '  no reference image found => exiting'
    RETURN
ENDIF
; calculate the center of gravity for the reference image
hcfg = rcfg->LocateByName('REFSTAR')
IF NOT OBJ_VALID(hcfg) THEN BEGIN
    PRINT, '  no reference stars in the reference image => exiting'
    RETURN
ENDIF
nstars = hcfg->GetNChilds()
refPos = FLTARR(2, nstars)
nr = 0
hhcfg = hcfg->GetFirst()
WHILE OBJ_VALID(hhcfg) DO BEGIN
    hhcfg->GetFloat2D, 'POS', x, y
    refPos[0, nr] = x
    refPos[1, nr] = y
    hhcfg = hhcfg->GetNext()
    nr++
ENDWHILE
refCX = MEAN(refPos[0,*])
refCY = MEAN(refPos[1,*])
PRINT, '  ref center: ', refCX, ', ', refCY
; calculate the center of gravity for the image
hcfg = icfg->LocateByName('REFSTAR')
IF NOT OBJ_VALID(hcfg) THEN BEGIN
    PRINT, '  no reference stars in the image => exiting'
    RETURN
ENDIF
IF nstars NE hcfg->GetNChilds() THEN BEGIN
    PRINT, '  different numbers of reference stars found => exiting'
    RETURN
ENDIF
imgPos = FLTARR(2, nstars)
nr = 0
hhcfg = hcfg->GetFirst()
WHILE OBJ_VALID(hhcfg) DO BEGIN
    hhcfg->GetFloat2D, 'POS', x, y
    imgPos[0, nr] = x
    imgPos[1, nr] = y
    hhcfg = hhcfg->GetNext()
    nr++
ENDWHILE
imgCX = MEAN(imgPos[0,*])
imgCY = MEAN(imgPos[1,*])
PRINT, '  img center: ', imgCX, ', ', imgCY
; calculate for each reference star the difference angle
; refAngle     : rotation angle of the reference image
; refPos[0,*]  : x coordinate of a reference star in the reference image
; refPos[1,*]  : y coordinate of a reference star in the reference image
; refCX, refCY : center of gravity of all reference stars in the reference image
; imgPos[0,*]  : x coordinate of a reference star in the current image
; imgPos[1,*]  : y coordinate of a reference star in the current image
; imgCX, imgCY : center of gravity of all reference stars in the current image
angle = 0.0
weight = 0.0
FOR nr=0,nstars-1 DO BEGIN
    dx = refPos[0,nr] - refCX
    dy = refPos[1,nr] - refCY
    d = SQRT(dx*dx + dy*dy)
    refAngle = ATAN(refPos[1,nr] - refCY, refPos[0,nr] - refCX)
    imgAngle = ATAN(imgPos[1,nr] - imgCY, imgPos[0,nr] - imgCX)
    diffAngle = imgAngle - refAngle
    IF diffAngle GT  !PI THEN diffAngle -= 2.0*!PI
    IF diffAngle LT -!PI THEN diffAngle += 2.0*!PI
    PRINT, '  star[', nr, ']: refAngle=', refAngle, ', imgAngle=', imgAngle, ', diffAngle=', diffAngle*!RADEG
    angle += d*diffAngle
    weight += d
ENDFOR
angle /= weight
r = -(angle*!RADEG - refRot)
PRINT, FORMAT='("  => rotate=", F8.4)', r
END
