PRO LDRS_ESTIMATE_TRANSLATION_FROM_REFSTARS, calFT, fileNr, imgNr, tag, r, sx, sy, tx, ty
PRINT, 'LDRS_TRANSLATION_FROM_REFSTARS, .., ', fileNr, ', ', imgNr, ', ...'
IF NOT OBJ_VALID(calFT) THEN BEGIN
    PRINT, '  calFT is not an object => exiting'
    RETURN
ENDIF
file = calFT->GetFile(fileNr)
IF NOT OBJ_VALID(file) THEN BEGIN
    PRINT, '  file ', fileNr, ' not found => exiting'
    RETURN
ENDIF
res = file->LoadData()
nx = file->GetNX()
ny = file->GetNY()
nImages = file->GetNImages()
; 1. find the configuration of the image and store it in icfg
icfg = file->GetImageCFG(imgNr)
IF NOT OBJ_VALID(icfg) THEN BEGIN
    PRINT, '  cfg for image ', imgNr, ' not found => exiting'
    RETURN
ENDIF
isRef = icfg->GetBoolean('ISREF', DEFAULT=0)
IF isRef THEN BEGIN
    PRINT, '  image is the reference image => nothing changed'
    RETURN ; do not change the values for the reference image
ENDIF
; 2. find the reference image and store the configuration in rcfg
rcfg = OBJ_NEW()
refAngle = 0.0
refTX = 0.0
refTY = 0.0
FOR nr=0,nImages-1 DO BEGIN
    cfg = file->GetImageCFG(nr)
    IF NOT OBJ_VALID(cfg) THEN CONTINUE
    flag = cfg->GetBoolean('ISREF', DEFAULT=0)
    If flag THEN BEGIN
        rcfg = cfg
        hcfg = cfg->LocateByName(tag, /NO_CREATE)
        IF OBJ_VALID(hcfg) THEN BEGIN
            refAngle = hcfg->GetFloat('ROTATE', DEFAULT=0.0)
            hcfg->GetFloat2D, 'TRANSLATE', refTX, refTY
        ENDIF
        BREAK
    ENDIF
ENDFOR
IF NOT OBJ_VALID(rcfg) THEN BEGIN
    PRINT, '  no reference image found => exiting'
    RETURN
ENDIF
; calculate the center of gravity for the reference image
hcfg = rcfg->LocateByName('REFSTAR')
IF NOT OBJ_VALID(hcfg) THEN BEGIN
    PRINT, '  no reference stars in the reference image => exiting'
    RETURN
ENDIF
nstars = hcfg->GetNChilds()
refPos = FLTARR(2, nstars)
nr = 0
hhcfg = hcfg->GetFirst()
WHILE OBJ_VALID(hhcfg) DO BEGIN
    hhcfg->GetFloat2D, 'POS', x, y
    refPos[0, nr] = x - nx/2
    refPos[1, nr] = y - ny/2
    hhcfg = hhcfg->GetNext()
    nr++
ENDWHILE
; calculate the center of gravity for the image
hcfg = icfg->LocateByName('REFSTAR')
IF NOT OBJ_VALID(hcfg) THEN BEGIN
    PRINT, '  no reference stars in the image => exiting'
    RETURN
ENDIF
IF nstars NE hcfg->GetNChilds() THEN BEGIN
    PRINT, '  different numbers of reference stars found => exiting'
    RETURN
ENDIF
imgPos = FLTARR(2, nstars)
transX = FLTARR(nstars)
transY = FLTARR(nstars)
nr = 0
hhcfg = hcfg->GetFirst()
WHILE OBJ_VALID(hhcfg) DO BEGIN
    hhcfg->GetFloat2D, 'POS', x, y
    imgPos[0, nr] = x - nx/2
    imgPos[1, nr] = y - ny/2
    hhcfg = hhcfg->GetNext()
    nr++
ENDWHILE
; calculate the translation vector, keep the rotation angle fixed
; refAngle     : rotation angle of the reference image
; refPos[0,*]  : x coordinate of a reference star in the reference image
; refPos[1,*]  : y coordinate of a reference star in the reference image
; imgPos[0,*]  : x coordinate of a reference star in the current image
; imgPos[1,*]  : y coordinate of a reference star in the current image</i>
phi = !DTOR*(r - refAngle)
FOR nr=0, nstars-1 DO BEGIN
    xs = imgPos[0,nr]*COS(phi) - imgPos[1,nr]*SIN(phi)
    ys = imgPos[0,nr]*SIN(phi) + imgPos[1,nr]*COS(phi)
    transX[nr] = refPos[0,nr] - xs
    transY[nr] = refPos[1,nr] - ys
    PRINT, FORMAT='("  ref=(", F8.2, ",", F8.2, "), img= (", F8.2, ",", F8.2, "), rot=(", F8.2, ",", F8.2, "), trans=(", F8.3, ",", F8.3, ")")', $
      refPos[0,nr], refPos[1,nr], imgPos[0,nr], imgPos[1,nr], xs, ys, transX[nr], transY[nr]
ENDFOR
tx = MEAN(transX)
ty = MEAN(transY)
PRINT, FORMAT='("  => translate=(", F8.2, ",", F8.2, ")")', tx, ty
END
