;; Check if all files of a file type are available:
;; If this file type is only the input of a module, check if all files
;; do exist.
;; If this file type is also the output of a module, check if this
;; module is used and was executed and all files do exist
;FUNCTION LDRS_PIPELINE_CHECK_FILETYPE, ft
;COMMON ldrs, env
;; check if this file type is already marked with a 1
;IF ft->GetMark() EQ 1 THEN RETURN, 1
;; check if this module is the result of an executed module
;outFlag = 0
;nModules = env.recipe->GetNModules()
;FOR iModule=0,nModules-1 DO BEGIN
;    module = env.recipe->GetModule(iModule)
;    ; check if the module is used, if not -> ignore
;    IF module->IsUsed() EQ 0 THEN CONTINUE
;    ; check if the module is executed, if not -> ignore
;    IF module->GetMark() EQ 0 THEN CONTINUE
;    nOut = module->GetNOutFiletypes()
;    FOR iOut=0,nOut-1 DO BEGIN
;        oft = module->GetOutFiletype(iOut)
;        IF ft EQ oft THEN BEGIN
;            outFlag = 1
;            BREAK
;        ENDIF
;    ENDFOR
;    ; check if this module uses the file type
;    IF outFlag
;ENDFOR
;END

; Execute a recipe in the following way:
;  1. If CFG is given:
;     => load the new configuration file (depends on FORCE)
;  2. If there is a loaded recipe and the name is not equal to the
;     argument (RECIPE):
;     => load the given recipe from file
;  3. The execution depends if one or more processing steps are given
;    a. If at least one processing step is given (PROC):
;       => they are executed in the given order
;    b. No processing step are given:
;       => mark all modules with a 0 (= not executed)
;       => mark all file types which are only input
;          with a 0 (= not available) or 1 (= available)
;       => mark all unused modules with a 1 (= executed)
;       ...
PRO LDRS_PIPELINE, RECIPE=recipeArg, PROC=procArg, CFG=cfg, FORCE=force
COMMON ldrs, env
LDRS_ENV, CFG=cfg, FORCE=force
IF N_ELEMENTS(recipeArg) NE 0 THEN BEGIN
    ; check if the parameter is a string
    IF SIZE(recipeArg, /TYPE) NE 7 THEN BEGIN
        PRINT, 'the recipe must be a string (file name of a recipe file)!'
        RETURN
    ENDIF
    IF env.recipe->GetName() NE recipeArg THEN BEGIN
        ; read a different recipe from file
        env.recipe->Read, recipeArg
    ENDIF
ENDIF
IF N_ELEMENTS(procArg) NE 0 THEN BEGIN
    IF SIZE(procArg, /TYPE) EQ 7 THEN procArg = [procArg]
    nProc = N_ELEMENTS(procArg)
    FOR iProc=0,nProc-1 DO BEGIN
        m = env.recipe->FindModule(procArg[iProc])
        IF NOT OBJ_VALID(m) THEN BEGIN
            PRINT, 'LDRS_PIPELINE: could not find a module with a tag or label as ', procArg[iProc]
            PRINT, '  valid tags or labels are:'
            nMod = env.recipe->GetNModules()
            FOR iMod=0,nMod-1 DO BEGIN
                m = env.recipe->GetModule(iMod)
                PRINT, '    ', m->GetTag(), ', ', m->GetLabel()
            ENDFOR
            RETURN
        ENDIF
        a = m->GetSelectedAlgorithm()
        IF NOT OBJ_VALID(a) THEN BEGIN
            PRINT, 'LDRS_PIPELINE: no algorithm selected for module with the label ', m->GetLabel()
            RETURN
        ENDIF
        a->Execute
    ENDFOR
    RETURN  ; leave the pipeline
ENDIF
END
