FUNCTION ldrsCALIBRATE::Init, algorithm
IF NOT self->ldrsALGOIMPL::Init(algorithm) THEN RETURN, 0
self.dlgTitle = 'Calibration Setup'
self.rawFT = self.recipe->FindFiletype('RAW')
self.skyFT = self.recipe->FindFiletype('SKY')
self.bpmFT = self.recipe->FindFiletype('BPM')
self.ffmFT = self.recipe->FindFiletype('FFM')
self.calFT = self.recipe->FindFiletype('CAL')
self.useSKY = 0
self.useFFM = 0
RETURN, 1
END

FUNCTION ldrsCALIBRATE::Prepare
;PRINT, 'ldrsCALIBRATE::Prepare'
IF NOT self->ldrsALGOIMPL::Prepare() THEN RETURN, 0
IF NOT self->CheckInputFiles(self.rawFT, 1, 10) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.skyFT, 0, 1) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.bpmFT, 0, 1) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.ffmFT, 0, 1) THEN RETURN, 0
nFiles = self.rawFT->GetNFiles()
IF NOT self->CheckOutputFiles(self.calFT, nFiles, nFiles) THEN RETURN, 0
nFiles = self.skyFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.skyFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
ENDFOR
nFiles = self.bpmFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.bpmFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
ENDFOR
nFiles = self.ffmFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.ffmFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
ENDFOR
RETURN, 1
END

FUNCTION ldrsCALIBRATE::Execute
;PRINT, 'ldrsCALIBRATE::Execute'
IF NOT self->ldrsALGOIMPL::Execute() THEN RETURN, 0
nFiles = self.rawFT->GetNFiles()
nSKY = self.skyFT->GetNFiles()
IF (nSKY EQ 1) AND self.useSKY THEN BEGIN
    file = self.skyFT->GetFile(0)
    sky = file->GetData()
ENDIF
nFFM = self.ffmFT->GetNFiles()
IF (nFFM EQ 1) AND self.useFFM THEN BEGIN
    file = self.ffmFT->GetFile(0)
    ffm = file->GetData()
ENDIF
nBPM = self.bpmFT->GetNFiles()
IF (nBPM EQ 1) THEN BEGIN
    file = self.bpmFT->GetFile(0)
    bpm = file->GetData()
    bpmProc = LDRS_GET_PROCEDURE('BPMC', self.bpmAlgo)
ENDIF ELSE BEGIN
    bpmProc = 'LDRS_BPM_NONE'
ENDELSE
FOR iFile=0,nFiles-1 DO BEGIN
    rawFile = self.rawFT->GetFile(iFile)
    rawData = rawFile->LoadData()
    rawData = rawFile->GetData()
    calFile = self.calFT->GetFile(iFile)
    calFile->CreateData, rawFile->GetNX(), rawFile->GetNY(), rawFile->GetNZ()
    calData = calFile->GetData()
    nImages = rawFile->GetNImages()
    FOR iImage=0,nImages-1 DO BEGIN
        data = (*rawData)[*,*,iImage]
        IF (nSKY EQ 1) AND self.useSKY THEN data -= *sky
        IF (nFFM EQ 1) AND self.useFFM THEN data /= *ffm
        IF (nBPM EQ 1) THEN CALL_PROCEDURE, bpmProc, data, bpm
        (*calData)[*,*,iImage] = data
    ENDFOR
ENDFOR
RETURN, 1
END

FUNCTION ldrsCALIBRATE::Finish
COMMON ldrs, env
;PRINT, 'ldrsCALIBRATE::Finish'
IF NOT self->ldrsALGOIMPL::Finish() THEN RETURN, 0
nFiles = self.calFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.calFT->GetFile(iFile)
    IF file->StoreData() EQ 0 THEN RETURN, 0
ENDFOR
IF nFiles GT 1 THEN BEGIN
    file = self.calFT->GetFile(0)
    res = file->LoadData()
    env.recipe->Broadcast, 'FileLoaded', file
ENDIF
RETURN, 1
END

PRO ldrsCALIBRATE::SetBPMAlgo, id, item
self.bpmAlgo = item
END

PRO ldrsCALIBRATE::Load, cfg
self->ldrsALGOIMPL::Load, cfg
IF OBJ_VALID(cfg) THEN BEGIN
    ;PRINT, 'ldrsCALIBRATE::Load, a valid cfg found'
    self.useSKY  = cfg->GetInt('USESKY', DEFAULT=0)
    self.useFFM  = cfg->GetInt('USEFFM', DEFAULT=0)
    self.bpmAlgo = cfg->GetInt('BPMALGO', DEFAULT=0)
ENDIF ELSE BEGIN
    ;PRINT, 'ldrsCALIBRATE::Load, no valid cfg found'
    self.useSKY  = 0
    self.useFFM  = 0
    self.bpmAlgo = 0
ENDELSE
END

PRO ldrsCALIBRATE::Store, cfg
self->ldrsALGOIMPL::Store, cfg
IF OBJ_VALID(cfg) THEN BEGIN
    ;PRINT, 'ldrsCALIBRATE::Store, a valid cfg found'
    cfg->SetInt, 'USESKY',  self.useSKY
    cfg->SetInt, 'USEFFM',  self.useFFM
    cfg->SetInt, 'BPMALGO', self.bpmAlgo
ENDIF
END

PRO ldrsCALIBRATE::ReadDynamicContent
WIDGET_CONTROL, self.dUseSKY, SET_VALUE=[self.useSKY]
WIDGET_CONTROL, self.dUseFFM, SET_VALUE=[self.useFFM]
WIDGET_CONTROL, self.dBPMAlgo, SET_COMBOBOX_SELECT=self.bpmAlgo
END

PRO ldrsCALIBRATE::WriteDynamicContent
WIDGET_CONTROL, self.dUseSKY, GET_VALUE=flags
self.useSKY = flags[0]
WIDGET_CONTROL, self.dUseFFM, GET_VALUE=flags
self.useFFM = flags[0]
END

PRO ldrsCALIBRATE::CreateContent
self->ldrsALGOIMPL::CreateContent
row = self->GetFreeRow()
lab           = self->CreateLabel(0, row, 'SKY:', /HALIGN_RIGHT)
self.dUseSKY  = self->CreateCheckbox(1, row, 'Subtract', /HALIGN_LEFT)
row++
lab           = self->CreateLabel(0, row, 'Flatfield:', /HALIGN_RIGHT)
self.dUseFFM  = self->CreateCheckbox(1, row, 'Divide', /HALIGN_LEFT)
row++
lab           = self->CreateLabel(0, row, 'Bad Pixels:', /HALIGN_RIGHT)
self.dBPMAlgo = self->CreateProcedureMenu(1, row, 'BPMC', 'SetBPMAlgo')
row++
END

PRO ldrsCALIBRATE__define
struct = { ldrsCALIBRATE, $
           INHERITS ldrsALGOIMPL, $
           rawFT:OBJ_NEW(), $
           skyFT:OBJ_NEW(), $
           bpmFT:OBJ_NEW(), $
           ffmFT:OBJ_NEW(), $
           calFT:OBJ_NEW(), $
           dUseSKY:0, dUseFFM:0, $
           dBPMAlgo:0, bpmAlgo:0, $
           useSKY:0, useFFM:0 $
         }
END
