FUNCTION ldrsCFG::Init, parent, name, nr
  ;PRINT, 'ldrsCFG::Init, ', parent, ", '", name, "', ", nr
  IF nr EQ 0 THEN BEGIN
      IF OBJ_VALID(parent) THEN BEGIN
          plast = parent.last
          IF OBJ_VALID(plast) THEN nr = plast.nr + 1 ELSE nr = 1
      END
  END
  self.parent = parent
  self.name = name
  self.nr = nr
  IF OBJ_VALID(parent) THEN BEGIN
      IF OBJ_VALID(parent.first) THEN BEGIN
          plast = parent.last
          self.prev = plast
          plast.next = self
          parent.last = self
      END ELSE BEGIN
          parent.first = self
          parent.last = self
      END
  END
  IF nr NE -1 THEN BEGIN
      ; put this instance at the correct position (increasing numbers)
      WHILE OBJ_VALID(self.prev) DO BEGIN
          sprev = self.prev
          spprev = sprev.prev
          snext = self.next
          IF sprev.nr LT nr THEN BREAK
          IF OBJ_VALID(spprev) THEN spprev.next = self
          self.next = sprev
          sprev.next = snext
          IF OBJ_VALID(snext) THEN snext.prev = sprev
          sprev.prev = self
          self.prev = spprev
      ENDWHILE
      cfg = self
      WHILE OBJ_VALID(cfg.prev) DO cfg = cfg.prev
      parent.first = cfg
      cfg = self
      WHILE OBJ_VALID(cfg.next) DO cfg = cfg.next
      parent.last = cfg
  END
  RETURN, 1
END

PRO ldrsCFG::Cleanup
  WHILE OBJ_VALID(self.first) DO OBJ_DESTROY, self.first
  IF OBJ_VALID(self.parent) THEN BEGIN
      p = self.parent
      IF p.first EQ self THEN p.first = self.next
      IF p.last EQ self THEN p.last = self.prev
      sprev = self.prev
      snext = self.next
      IF OBJ_VALID(sprev) THEN sprev.next = snext
      IF OBJ_VALID(snext) THEN snext.prev = sprev
  END
END

FUNCTION ldrsCFG::Clone, parent=par
  ; first create a clone of myself
  IF NOT KEYWORD_SET(par) THEN par = OBJ_NEW()
  clone = OBJ_NEW('ldrsCFG', par, self.name, self.nr)
  ; set the value of the clone
  clone.type = self.type
  clone.iVal = self.iVal
  clone.fVal = self.fVal
  clone.sVal = self.sVal
  ; and now clone all children
  child = self.first
  WHILE OBJ_VALID(child) DO BEGIN
      hclone = child->Clone(PARENT=clone)
      child = child.next
  ENDWHILE
  RETURN, clone
END

PRO ldrsCFG::CloneChildren, target
; and now clone all children
child = self.first
WHILE OBJ_VALID(child) DO BEGIN
    hclone = child->Clone(PARENT=target)
    child = child.next
ENDWHILE
END

PRO ldrsCFG::MoveChildren, target
target.first = self.first
target.last = self.last
child = self.first
WHILE OBJ_VALID(child) DO BEGIN
    child.parent = target
    child = child.next
ENDWHILE
self.first = OBJ_NEW()
self.last = OBJ_NEW()
END

FUNCTION ldrsCFG::LocateByNr, nr, no_create=no_cr
  IF NOT KEYWORD_SET(no_cr) THEN no_cr = 0
  child = self.first
  WHILE OBJ_VALID(child) DO BEGIN
      IF child.nr EQ nr THEN BREAK
      child = child.next
  ENDWHILE
  IF NOT OBJ_VALID(child) AND no_cr EQ 0 THEN BEGIN
      child = OBJ_NEW('ldrsCFG', self, '', nr)
  END
  RETURN, child
END

FUNCTION ldrsCFG::LocateByName, name, no_create=no_cr
  IF NOT KEYWORD_SET(no_cr) THEN no_cr = 0
  ;PRINT, '  ldrsCFG::LocateByName, ', name, ', NO_CREATE=', no_cr
  child = self.first
  WHILE OBJ_VALID(child) DO BEGIN
      IF child.name EQ name THEN BREAK
      child = child.next
  ENDWHILE
  IF NOT OBJ_VALID(child) AND no_cr EQ 0 THEN BEGIN
      child = OBJ_NEW('ldrsCFG', self, name, -1)
  END
  RETURN, child
END

FUNCTION ldrsCFG::LocateByValue, prefix, name, value
  ;PRINT, 'ldrsCFG::LocateByValue, ', prefix, ', ', name, ', ', value
  place = self->Locate(prefix)
  IF NOT OBJ_VALID(place) THEN RETURN, OBJ_NEW()
  child = place.first
  WHILE OBJ_VALID(child) DO BEGIN
      node = child->Locate(name)
      IF node.sVal EQ value THEN RETURN, child
      child = child.next
  ENDWHILE
  RETURN, OBJ_NEW()
END

FUNCTION ldrsCFG::Locate, qName, no_create=no_cr
  IF NOT KEYWORD_SET(no_cr) THEN no_cr = 0
  ;PRINT, '  ldrsCFG::Locate, ', qName, ', NO_CREATE=', no_cr
  IF qName EQ '' THEN RETURN, self
  curr = self
  list = STRSPLIT(qName, '.', /EXTRACT)
  FOR nr = 0, N_ELEMENTS(list) - 1 DO BEGIN
      ;PRINT, FORMAT='("  part #", I0, "=", A)', nr, list[nr]
      name=STREGEX(list[nr], '^[A-Z_]+', /EXTRACT)
      curr = curr->LocateByName(name, no_create=no_cr)
      IF NOT OBJ_VALID(curr) THEN RETURN, curr
      IF STREGEX(list[nr], '^[A-Z_]+[0-9]+$', /BOOLEAN) THEN BEGIN
          num =FIX(STREGEX(list[nr], '[0-9]+$', /EXTRACT))
          ;PRINT, FORMAT='("    contains the name ", A, " and the number ", I0)', name, num
          curr = curr->LocateByNr(num, no_create=no_cr)
          IF NOT OBJ_VALID(curr) THEN RETURN, curr
      END
  ENDFOR
  RETURN, curr
END

PRO ldrsCFG::SetInt, qName, value
  place = self->Locate(qName)
  place.type = 1
  place.iVal = value
END

FUNCTION ldrsCFG::GetInt, qName, default=def
  IF NOT KEYWORD_SET(def) THEN def = 0
  place = self->Locate(qName, /NO_CREATE)
  IF NOT OBJ_VALID(place) THEN RETURN, def
  CASE place.type OF
      1:RETURN, place.iVal
      2:RETURN, FIX(place.fVal)
      3: BEGIN
          place.type = 1
          place.iVal = FIX(place.sVal)
          RETURN, place.iVal
      END
      ELSE: RETURN, 0
  ENDCASE
END

PRO ldrsCFG::SetInt2D, qName, x, y
  place = self->Locate(qName)
  place.type = 3
  place.sVal = STRING(FORMAT='(I0, " ", I0)', x, y)
END

PRO ldrsCFG::GetInt2D, qName, x, y, default=def
IF KEYWORD_SET(def) THEN BEGIN
    x = def[0]
    y = def[1]
ENDIF ELSE BEGIN
    x = 0
    y = 0
ENDELSE
place = self->Locate(qName, /NO_CREATE)
IF NOT OBJ_VALID(place) THEN RETURN
CASE place.type OF
    1: BEGIN
        x = place.iVal
        y = x
    END
    2: BEGIN
        x = FIX(place.fVal)
        y = x
    END
    3: BEGIN
        entries = STRSPLIT(place.sVal, ' ', /EXTRACT)
        IF N_ELEMENTS(entries) NE 2 THEN RETURN
        x = FIX(entries[0])
        y = FIX(entries[1])
    END
    ELSE: RETURN
ENDCASE
END

PRO ldrsCFG::SetInt3D, qName, x, y, z
  place = self->Locate(qName)
  place.type = 3
  place.sVal = STRING(FORMAT='(I0, " ", I0, " ", I0)', x, y, z)
END

PRO ldrsCFG::GetInt3D, qName, x, y, z, default=def
IF KEYWORD_SET(def) THEN BEGIN
    x = def[0]
    y = def[1]
    z = def[2]
ENDIF ELSE BEGIN
    x = 0
    y = 0
    z = 0
ENDELSE
place = self->Locate(qName, /NO_CREATE)
IF NOT OBJ_VALID(place) THEN RETURN
CASE place.type OF
    1: BEGIN
        x = place.iVal
        y = x
        z = x
    END
    2: BEGIN
        x = FIX(place.fVal)
        y = x
        z = x
    END
    3: BEGIN
        entries = STRSPLIT(place.sVal, ' ', /EXTRACT)
        IF N_ELEMENTS(entries) NE 3 THEN RETURN
        x = FIX(entries[0])
        y = FIX(entries[1])
        z = FIX(entries[2])
    END
    ELSE: RETURN
ENDCASE
END

PRO ldrsCFG::SetInt4D, qName, x, y, z, t
  place = self->Locate(qName)
  place.type = 3
  place.sVal = STRING(FORMAT='(I0, " ", I0, " ", I0, " ", I0)', x, y, z, t)
END

PRO ldrsCFG::GetInt4D, qName, x, y, z, t, default=def
IF KEYWORD_SET(def) THEN BEGIN
    x = def[0]
    y = def[1]
    z = def[2]
    t = def[3]
ENDIF ELSE BEGIN
    x = 0
    y = 0
    z = 0
    t = 0
ENDELSE
place = self->Locate(qName, /NO_CREATE)
IF NOT OBJ_VALID(place) THEN RETURN
CASE place.type OF
    1: BEGIN
        x = place.iVal
        y = x
        z = x
        t = x
    END
    2: BEGIN
        x = FIX(place.fVal)
        y = x
        z = x
        t = x
    END
    3: BEGIN
        entries = STRSPLIT(place.sVal, ' ', /EXTRACT)
        IF N_ELEMENTS(entries) NE 4 THEN RETURN
        x = FIX(entries[0])
        y = FIX(entries[1])
        z = FIX(entries[2])
        t = FIX(entries[3])
    END
    ELSE: RETURN
ENDCASE
END

PRO ldrsCFG::SetFloat, qName, value
  place = self->Locate(qName)
  place.type = 2
  place.fVal = value
END

FUNCTION ldrsCFG::GetFloat, qName, default=def
  IF NOT KEYWORD_SET(def) THEN def = 0.0
  place = self->Locate(qName, /NO_CREATE)
  IF NOT OBJ_VALID(place) THEN RETURN, def
  CASE place.type OF
      1:RETURN, FLOAT(place.iVal)
      2:RETURN, place.fVal
      3: BEGIN
          place.type = 2
          place.fVal = FLOAT(place.sVal)
          RETURN, place.fVal
      END
      ELSE: RETURN, 0.0
  ENDCASE
END

PRO ldrsCFG::SetFloat2D, qName, x, y
  place = self->Locate(qName)
  place.type = 3
  place.sVal = STRING(FORMAT='(F0, " ", F0)', x, y)
END

PRO ldrsCFG::GetFloat2D, qName, x, y, default=def
IF KEYWORD_SET(def) THEN BEGIN
    x = def[0]
    y = def[1]
ENDIF ELSE BEGIN
    x = 0.0
    y = 0.0
ENDELSE
place = self->Locate(qName, /NO_CREATE)
IF NOT OBJ_VALID(place) THEN RETURN
CASE place.type OF
    1: BEGIN
        x = FLOAT(place.iVal)
        y = x
    END
    2: BEGIN
        x = place.fVal
        y = x
    END
    3: BEGIN
        entries = STRSPLIT(place.sVal, ' ', /EXTRACT)
        IF N_ELEMENTS(entries) NE 2 THEN RETURN
        x = FLOAT(entries[0])
        y = FLOAT(entries[1])
    END
    ELSE: RETURN
ENDCASE
END

PRO ldrsCFG::SetFloat3D, qName, x, y, z
  place = self->Locate(qName)
  place.type = 3
  place.sVal = STRING(FORMAT='(F0, " ", F0, " ", F0)', x, y, z)
END

PRO ldrsCFG::GetFloat3D, qName, x, y, z, default=def
IF KEYWORD_SET(def) THEN BEGIN
    x = def[0]
    y = def[1]
    z = def[2]
ENDIF ELSE BEGIN
    x = 0.0
    y = 0.0
    z = 0.0
ENDELSE
place = self->Locate(qName, /NO_CREATE)
IF NOT OBJ_VALID(place) THEN RETURN
CASE place.type OF
    1: BEGIN
        x = FLOAT(place.iVal)
        y = x
        z = x
    END
    2: BEGIN
        x = place.fVal
        y = x
        z = x
    END
    3: BEGIN
        entries = STRSPLIT(place.sVal, ' ', /EXTRACT)
        IF N_ELEMENTS(entries) NE 3 THEN RETURN
        x = FLOAT(entries[0])
        y = FLOAT(entries[1])
        z = FLOAT(entries[2])
    END
    ELSE: RETURN
ENDCASE
END

PRO ldrsCFG::SetFloat4D, qName, x, y, z, t
  place = self->Locate(qName)
  place.type = 3
  place.sVal = STRING(FORMAT='(F0, " ", F0, " ", F0, " ", F0)', x, y, z, t)
END

PRO ldrsCFG::GetFloat4D, qName, x, y, z, t, default=def
IF KEYWORD_SET(def) THEN BEGIN
    x = def[0]
    y = def[1]
    z = def[2]
    t = def[3]
ENDIF ELSE BEGIN
    x = 0.0
    y = 0.0
    z = 0.0
    t = 0.0
ENDELSE
place = self->Locate(qName, /NO_CREATE)
IF NOT OBJ_VALID(place) THEN RETURN
CASE place.type OF
    1: BEGIN
        x = FLOAT(place.iVal)
        y = x
        z = x
        t = x
    END
    2: BEGIN
        x = place.fVal
        y = x
        z = x
        t = x
    END
    3: BEGIN
        entries = STRSPLIT(place.sVal, ' ', /EXTRACT)
        IF N_ELEMENTS(entries) NE 4 THEN RETURN
        x = FLOAT(entries[0])
        y = FLOAT(entries[1])
        z = FLOAT(entries[2])
        t = FLOAT(entries[3])
    END
    ELSE: RETURN
ENDCASE
END

PRO ldrsCFG::SetFloatN, qName, x
place = self->Locate(qName)
place.type = 3
s = STRARR(N_ELEMENTS(x))
FOR i = 0, N_ELEMENTS(s) - 1 DO BEGIN
    s[i] = STRING(x[i], FORMAT='(F0)')
ENDFOR
place.sVal = STRJOIN(s, ',', /SINGLE)
END

PRO ldrsCFG::GetFloatN, qName, x, default=def
IF N_ELEMENTS(def) NE 0 THEN x = def
place = self->Locate(qName, /NO_CREATE)
IF NOT OBJ_VALID(place) THEN RETURN
CASE place.type OF
    1: BEGIN
        x = [FLOAT(place.iVal)]
    END
    2: BEGIN
        x = [place.fVal]
    END
    3: BEGIN
        PRINT, 'GetFloatN : str=',place.sVal
        s = STRSPLIT(place.sVal, ',', /EXTRACT)
        x = FLTARR(N_ELEMENTS(s))
        FOR i = 0, N_ELEMENTS(s) - 1 DO BEGIN
            x[i] = FLOAT(s[i])
        ENDFOR
    END
    ELSE: RETURN
ENDCASE
END

PRO ldrsCFG::SetString, qName, value
  place = self->Locate(qName)
  place.type = 3
  place.sVal = value
END

FUNCTION ldrsCFG::GetString, qName, default=def
  IF NOT KEYWORD_SET(def) THEN def = ''
  ;PRINT, 'ldrsCFG::GetString, ', qName, ', DEFAULT=', def
  place = self->Locate(qName, /NO_CREATE)
  IF NOT OBJ_VALID(place) THEN RETURN, def
  CASE place.type OF
      1:RETURN, STRING(place.iVal)
      2:RETURN, STRING(place.fVal)
      3:RETURN, place.sVal
      ELSE: RETURN, ''
  ENDCASE
END

PRO ldrsCFG::SetBoolean, qName, value
place = self->Locate(qName)
place.type = 3
IF value EQ 0 THEN place.sVal = 'F' ELSE place.sVal = 'T'
END

FUNCTION ldrsCFG::GetBoolean, qName, default=def
IF NOT KEYWORD_SET(def) THEN def = 0
place = self->Locate(qName, /NO_CREATE)
IF NOT OBJ_VALID(place) THEN RETURN, def
CASE place.type OF
    1:RETURN, place.iVal
    2:RETURN, FIX(place.fVal)
    3: BEGIN
        IF place.sVal EQ 'true' THEN RETURN, 1
        IF place.sVal EQ 'TRUE' THEN RETURN, 1
        IF place.sVal EQ 'T' THEN RETURN, 1
        RETURN, 0
    END
    ELSE: RETURN, 0
ENDCASE
END

PRO ldrsCFG::Save, lun, prefix
  IF prefix EQ '' THEN BEGIN
      hprefix = self.name
  END ELSE BEGIN
      IF self.name NE '' THEN BEGIN
          hprefix = prefix + '.' + self.name
      END ELSE BEGIN
          hprefix = STRING(FORMAT='(A, I0)', prefix, self.nr)
      END
  END
  CASE self.type OF
      0: BEGIN
          child = self.first
          WHILE OBJ_VALID(child) DO BEGIN
              child->Save, lun, hprefix
              child = child.next
          ENDWHILE
      END
      1: PRINTF, lun, FORMAT='(A, " = ", A, I0, A)', hprefix, '"', self.iVal, '"'
      2: PRINTF, lun, FORMAT='(A, " = ", A, F0, A)', hprefix, '"', self.fVal, '"'
      3: PRINTF, lun, FORMAT='(A, " = ", A, A, A)', hprefix, '"', self.sVal, '"'
  ENDCASE
END

PRO ldrsCFG::Load, lun
  line = ''
  lineNr = 0
  WHILE NOT EOF(lun) DO BEGIN
      READF, lun, line
      ;PRINT, FORMAT='(" line ", I0, ": ", A)', lineNr, line
      lineNr = lineNr + 1
      IF STRLEN(line) LT 4 THEN CONTINUE  ; ignore empty lines
      IF STREGEX(line, '^#.*$', /BOOLEAN) THEN CONTINUE  ; ignore comment lines
      entries = STRSPLIT(line, '#', /EXTRACT)
      line = entries[0]
      IF STRLEN(line) LT 4 THEN CONTINUE  ; ignore empty lines
      ; process lines with double quotes around the value
      IF STREGEX(line, '.*=.*\".*\".*', /BOOLEAN) THEN BEGIN
          entries = STRSPLIT(line, '=', /EXTRACT)
          IF N_ELEMENTS(entries) NE 2 THEN CONTINUE ; ignore this line
          name = STRTRIM(entries[0], 2)
          value = STRTRIM(entries[1], 2)
          value = STRMID(value, 1, STRLEN(value) - 2)
          self->SetString, name, value
          CONTINUE
      END
      ; process lines without double quotes around the value
      IF STREGEX(line, '.*=.*', /BOOLEAN) THEN BEGIN
          entries = STRSPLIT(line, '=', /EXTRACT)
          IF N_ELEMENTS(entries) NE 2 THEN CONTINUE ; ignore this line
          name = STRTRIM(entries[0], 2)
          value = STRTRIM(entries[1], 2)
          self->SetString, name, value
          CONTINUE
      END
      ;PRINT, FORMAT='("    name=", A, ", value=", A)', name, value
  ENDWHILE
END

FUNCTION ldrsCFG::GetParent
RETURN, self.parent
END

FUNCTION ldrsCFG::GetFirst
  RETURN, self.first
END

FUNCTION ldrsCFG::GetLast
  RETURN, self.last
END

FUNCTION ldrsCFG::GetNext
  RETURN, self.next
END

FUNCTION ldrsCFG::GetNChilds
  n = 0
  child = self.first
  WHILE OBJ_VALID(child) DO BEGIN
      n = n + 1
      child = child.next
  ENDWHILE
  RETURN, n
END

FUNCTION ldrsCFG::GetName
RETURN, self.name
END

FUNCTION ldrsCFG::GetNr
RETURN, self.nr
END

PRO ldrsCFG::Debug
  HELP, self, /OBJECTS
END

PRO ldrsCFG__define
  struct = { ldrsCFG, $
             name:'', $
             nr:-1, $
             type:0, $
             parent:OBJ_NEW(), $
             next:OBJ_NEW(), $
             prev:OBJ_NEW(), $
             first:OBJ_NEW(), $
             last:OBJ_NEW(), $
             iVal:0, $
             fVal:0.0, $
             sVal:'' $
  }
END
