FUNCTION ldrsDECONV_RL::Init, algorithm
IF NOT self->ldrsDECONVOLVE::Init(algorithm) THEN RETURN, 0

RETURN, 1
END

FUNCTION ldrsDECONV_RL::Prepare
PRINT, 'ldrsDECONV_RL::Prepare'
IF NOT self->ldrsDECONVOLVE::Prepare() THEN RETURN, 0
PRINT, 'ldrsDECONV_RL::Prepare', self.nx, self.ny, self.nPA
s = FLOAT(self.nx)*FLOAT(self.ny)
; create and fill the psf arrays
self.psf = PTR_NEW(FLTARR(self.nx, self.ny, self.nPA))
self.psfFft = PTR_NEW(COMPLEXARR(self.nx, self.ny, self.nPA))
self.psfFftConj = PTR_NEW(COMPLEXARR(self.nx, self.ny, self.nPA))
nFiles = self.psfFT->GetNFiles()
iPA = 0
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.psfFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
    data = file->GetData()
    FOR iZ=0, file->GetNZ()-1 DO BEGIN
        (*self.psf)[*,*,iPA] = (*data)[*,*,iZ]
        iPA++
    ENDFOR
ENDFOR
index = WHERE(*self.psf LT 0.0, count)
IF count NE 0 THEN (*self.psf)[index] = 0.0
FOR iPA=0,self.nPA-1 DO BEGIN
    (*self.psf)[*,*,iPA] /= TOTAL((*self.psf)[*,*,iPA]) ; normalize it to integral 1.0
    (*self.psfFft)[*,*,iPA] = FFT((*self.psf)[*,*,iPA])
    (*self.psfFftConj)[*,*,iPA] = CONJ((*self.psfFft)[*,*,iPA])
ENDFOR
; fill the obj array
self.obj = PTR_NEW(FLTARR(self.nx, self.ny, self.nPA))
nFiles = self.objFT->GetNFiles()
iPA = 0
self.nPhot = 0.0
IF PTR_VALID(self.bgrd) THEN PTR_FREE, self.bgrd
self.bgrd = PTR_NEW(FLTARR(self.nPA))
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.objFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
    data = file->GetData()
    FOR iZ=0, file->GetNZ()-1 DO BEGIN
        (*self.obj)[*,*,iPA] = (*data)[*,*,iZ]
        cfg = file->GetImageCFG(iZ)
        IF OBJ_VALID(cfg) THEN BEGIN
            (*self.bgrd)[iPA] = cfg->GetFloat('BGRD')
            PRINT, '  background at ', iPA, ' = ', (*self.bgrd)[iPA]
        ENDIF
        self.nPhot += TOTAL((*data)[*,*,iZ]) - s*(*self.bgrd)[iPA]
        iPA++
    ENDFOR
ENDFOR
self.nPhot /= self.nPA
PRINT, '  self.nPhot=', self.nPhot
; create the corrections array
self.corr = PTR_NEW(FLTARR(self.nx, self.ny))
PRINT, 'ldrsDECONV_RL::Prepare', self.nx, self.ny, self.nPA, self.nPhot, self.nPhot/s
(*self.rec)[*] = self.nPhot/s
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.rec, NAME='Reconstruction'
;WAIT, 10
RETURN, 1
END

FUNCTION ldrsDECONV_RL::ExecuteStep
;PRINT, 'ldrsDECONV_RL::ExecuteStep: self.iIter=', self.iIter
IF NOT self->ldrsDECONVOLVE::ExecuteStep() THEN RETURN, 0
currectExcept = !EXCEPT
!EXCEPT = 0
void = CHECK_MATH()
s = 1.0/(FLOAT(self.nx)*FLOAT(self.ny))
(*self.corr) = 0.0
imax = MAX(*self.rec)
;PRINT, '   rec max=', imax
recFft = FFT(*self.rec)
FOR iPA=0,self.nPA-1 DO BEGIN
    tmpArr = recFft*((*self.psfFft)[*,*,iPA])
    tmpArr = SHIFT(ABS(FFT(tmpArr, /INVERSE)), self.nx/2, self.ny/2)
    ;tmpArr = ABS(FFT(tmpArr,
    ;/INVERSE))*(FLOAT(self.nx)*FLOAT(self.ny))
    tot = TOTAL(tmpArr)
    tmpArr *= self.nPhot/tot
    imax = MAX(tmpArr)
    tot = TOTAL(tmpArr)
    ;PRINT, '   img max=', imax, ' tot=', tot
    ;tmpArr = ABS(FFT(tmpArr, /INVERSE))
    ;hp1 = PTR_NEW(tmpArr)
    ;LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=hp1, NAME='1. tmpArr'
    ;WAIT, 5
    tmpArr += (*self.bgrd)[iPA]
    tmpArr = FFT((*self.obj)[*,*,iPA]/tmpArr[*,*])*(*self.psfFftConj)[*,*,iPA]
    tmpArr = SHIFT(ABS(FFT(tmpArr, /INVERSE)), self.nx/2, self.ny/2)
    ;tmpArr = ABS(FFT(tmpArr, /INVERSE))
    imax = MAX(tmpArr)
    ;PRINT, '   imax3=', imax
    ;hp2 = PTR_NEW(tmpArr)
    ;LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=hp2, NAME='2. tmpArr'
    ;WAIT, 5
    (*self.corr) += tmpArr
ENDFOR
(*self.corr) /= FLOAT(self.nPA)
;icorrmin = MIN(*self.corr)
;icorrmax = MAX(*self.corr)
;irecmin = MIN(*self.rec)
;irecmax = MAX(*self.rec)
;PRINT, '   corr= [', icorrmin, ' ... ', icorrmax, '], rec= [', irecmin, ' ... ', irecmax, ']'
;LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.corr, NAME='Correction'
;WAIT, 1
;(*self.rec) *= SHIFT((*self.corr), self.nx/2, self.ny/2)
(*self.rec) *= (*self.corr)
;(*self.rec) = ABS((*self.rec))
cs = TOTAL((*self.rec))
(*self.rec) *= self.nPhot/cs
; clip all values below 1e-20 photons to 0.0 (avoid floating point underflows)
IF (self.iIter MOD 10) EQ 0 THEN BEGIN
    ri = WHERE((*self.rec) LE 1.0E-20, count)
    IF count NE 0 THEN (*self.rec)[ri] = 0.0
ENDIF
status = CHECK_MATH()
IF (status AND NOT 32) NE 0 THEN MESSAGE, 'IDL CHECK_MATH() error: ' + STRTRIM(status, 2)
!EXCEPT = currectExcept
;PRINT, 'ldrsDECONV_RL::ExecuteStep, N_ELEMENTS((*self.rec))=', N_ELEMENTS((*self.rec))
;PRINT, 'ldrsDECONV_RL::ExecuteStep, SIZE((*self.rec))=', SIZE((*self.rec))
RETURN, 1
END

PRO ldrsDECONV_RL__define
struct = { ldrsDECONV_RL, $
           INHERITS ldrsDECONVOLVE, $
           psf:PTR_NEW(), $
           psfFft:PTR_NEW(), $
           psfFftConj:PTR_NEW(), $
           obj:PTR_NEW(), $
           corr:PTR_NEW(), $
           bgrd:PTR_NEW() $
         }
END
