FUNCTION ldrsDECONV_SBB::Init, algorithm
IF NOT self->ldrsBUILDINGBLOCK::Init(algorithm) THEN RETURN, 0

RETURN, 1
END

FUNCTION ldrsDECONV_SBB::Prepare
;PRINT, 'ldrsDECONV_SBB::Prepare'
IF NOT self->ldrsBUILDINGBLOCK::Prepare() THEN RETURN, 0
; reduce the set of object images to one image
file = self.objFT->GetFile(0)
objImg = FLTARR(self.nx, self.ny)
data = file->GetData()
objImg += *data
self.nphot = TOTAL(objImg) ; store the number of photons in order to rescale the result
objImg[*] /= self.nphot ; normalize the object image to an integral of 1.0
self.obj = PTR_NEW(objImg)
PRINT, ' total(obj)=', TOTAL(*self.obj)
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.obj, NAME='Object'
WAIT, 2
; and now reduce the set of psf images to one image
file = self.psfFT->GetFile(0)
psfImg = FLTARR(self.nx, self.ny)
data = file->GetData()
psfImg += *data
psfImg[*] /= TOTAL(psfImg) ; normalize the psf to an integral of 1.0
self.psf = PTR_NEW(psfImg)
PRINT, ' total(psf)=', TOTAL(*self.psf)
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.psf, NAME='Psf'
WAIT, 2
; calculate the FFT for the psf
self.psfFft = PTR_NEW(FFT(*self.psf))
self.psfFftAbs = PTR_NEW(ABS(*self.psfFft))
PRINT, 'psfFftAbs=', (*self.psfFftAbs)[0,0], (*self.psfFftAbs)[1,0], (*self.psfFftAbs)[1,1]
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.psfFftAbs, NAME='Psf FFT Absolute'
WAIT, 2
;self.psfFftConj = PTR_NEW(CONJ(*self.psfFft))
(*self.rec)[*] = 0.0
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.rec, NAME='Reconstruction'
WAIT, 2
RETURN, 1
END

FUNCTION ldrsDECONV_SBB::ExecuteStep
;PRINT, 'ldrsDECONV_SBB::ExecuteStep: self.iIter=', self.iIter
IF NOT self->ldrsBUILDINGBLOCK::ExecuteStep() THEN RETURN, 0
;PRINT, ' total(rec)=', TOTAL(*self.rec)
;peak = MAX(*self.rec)
;indices = WHERE((*self.rec) GT 0.0)
;PRINT, '    old blocks:'
;PRINT, '       peak=', peak
;PRINT, '       nblocks=', N_ELEMENTS(indices), ', indices=', indices
;tmpArr = ABS(FFT(*self.rec))
;LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=PTR_NEW(tmpArr), NAME='abs(fft(rec))'
;WAIT, 5
tmpArr = FFT(*self.rec)*(*self.psfFft)
htmpArr = ABS(FFT(tmpArr, /INVERSE))
htmpArr = SHIFT(htmpArr, self.nx/2, self.ny/2)
;LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=PTR_NEW(htmpArr), NAME='Reconstruction2'
;WAIT, 5
tot = TOTAL(htmpArr)
;PRINT, '    total=', tot
IF tot NE 0.0 THEN htmpArr[*] /= -tot
htmpArr += *self.obj
;LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=PTR_NEW(htmpArr), NAME='Difference'
;WAIT, 5
peak = MAX(htmpArr)
indices = WHERE(htmpArr GE 0.01*self.threshold*peak)
;PRINT, '    new blocks:'
;PRINT, '       peak=', peak, ', nblocks=', N_ELEMENTS(indices), ', first index=', indices[0]
;PRINT, '       indices=', indices
(*self.rec)[indices] += 1.0
;PRINT, ' total(rec)=', TOTAL(*self.rec)
RETURN, 1
END

PRO ldrsDECONV_SBB__define
struct = { ldrsDECONV_SBB, $
           INHERITS ldrsBUILDINGBLOCK, $
           psf:PTR_NEW(), $
           psfFft:PTR_NEW(), $
           psfFftConj:PTR_NEW(), $
           psfFftAbs:PTR_NEW(), $
           obj:PTR_NEW() $
         }
END
