FUNCTION ldrsDECONVOLVE::Init, algorithm
IF NOT self->ldrsALGOIMPL::Init(algorithm) THEN RETURN, 0
self.dlgTitle = 'Deconvolution Setup'
self.objFT = self.recipe->FindFiletype('TAR')
self.psfFT = self.recipe->FindFiletype('PSF')
self.rpsfFT = self.recipe->FindFiletype('RPSF')
self.rimgFT = self.recipe->FindFiletype('RIMG')
self.recFT = self.recipe->FindFiletype('REC')
self.imgFT = self.recipe->FindFiletype('IMG')
self.logFT = self.recipe->FindFiletype('LOG')
; set the default parameters
self.nIter = 1000
self.pIter = 10
self.primresult = 1
; termination of the reconstruction
self.lastErrMeasure = 10.0
self.breakFlag = 0
self.primdist_selected = 0
self.secdist_selected = 0
RETURN, 1
END

FUNCTION ldrsDECONVOLVE::Prepare
;PRINT, '####################ldrsDECONVOLVE::Prepare'
IF NOT self->ldrsALGOIMPL::Prepare() THEN RETURN, 0
IF NOT self->CheckInputFiles(self.objFT, 1, 10) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.psfFT, 1, 10) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.rpsfFT, 0, 1) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.rimgFT, 0, 1) THEN RETURN, 0
IF self->GetNoMP() EQ 0 THEN BEGIN
    IF NOT self->CheckOutputFiles(self.recFT, 1, 1) THEN RETURN, 0
    IF NOT self->CheckOutputFiles(self.imgFT, 0, 1) THEN RETURN, 0
    IF NOT self->CheckOutputFiles(self.logFT, 0, 1) THEN RETURN, 0
ENDIF
self.nx = 0
self.ny = 0
self.nPA= 0
nFiles = self.objFT->GetNFiles()
lambda = 2.2
dlambda = 0.0
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.objFT->GetFile(iFile)
    cfg = file->GetFileCFG()
    hcfg = cfg->Locate('OBS.LAMBDA', /NO_CREATE)
    IF OBJ_VALID(hcfg) THEN lambda = hcfg->GetFloat('', DEFAULT=2.2)
    hcfg = cfg->Locate('OBS.DLAMBDA', /NO_CREATE)
    IF OBJ_VALID(hcfg) THEN dlambda = hcfg->GetFloat('', DEFAULT=0.0)
    IF file->LoadData() EQ 0 THEN RETURN, 0
    self.nx = file->GetNX()
    self.ny = file->GetNY()
    self.nPA += file->GetNZ()
ENDFOR
nFiles = self.psfFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.psfFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
ENDFOR
; get and load the optional reference psf
self.rpsfFile = OBJ_NEW()
nFiles = self.rpsfFT->GetNFiles()
IF nFiles EQ 1 THEN BEGIN
    self.rpsfFile = self.rpsfFT->GetFile(0)
    IF self.rpsfFile->LoadData() EQ 0 THEN RETURN, 0
ENDIF
; get and load the optional reference image
self.rimgFile = OBJ_NEW()
nFiles = self.rimgFT->GetNFiles()
IF nFiles EQ 1 THEN BEGIN
    self.rimgFile = self.rimgFT->GetFile(0)
    IF self.rimgFile->LoadData() EQ 0 THEN RETURN, 0
    self.lastOpt = PTR_NEW(FLTARR(self.nx, self.ny))
ENDIF


;creae the rec file according to the result output mode
rec_nz = 1
CASE self.primresult OF
   0: rec_nz = 1                                      ;last image
   1: rec_nz = self.nIter/self.pIter                  ;image cube
   2: rec_nz = 1                                      ;best image of selected distance
   3: rec_nz = N_ELEMENTS(*(self->Get_distances()))   ;image cube of all distances
ENDCASE

IF self->GetNoMP() EQ 0 THEN BEGIN
    ; here is only the code for algorithms without multiple-value parameters
    ; get and create the mandatory reconstruction file
    self.recFile = OBJ_NEW()
    nFiles = self.recFT->GetNFiles()
    IF nFiles EQ 1 THEN BEGIN
        self.recFile = self.recFT->GetFile(0)
        self.recFile->CreateData, self.nx, self.ny, rec_nz, /FORCE
        cfg = self.recFile->GetFileCFG()
        cfg->SetFloat, 'OBS.LAMBDA', lambda
        cfg->SetFloat, 'OBS.DLAMBDA', dlambda
    ENDIF
    ; get and create the optional image file
    nFiles = self.imgFT->GetNFiles()
    IF nFiles EQ 1 THEN BEGIN
        self.imgFile = self.imgFT->GetFile(0)
        self.imgFile->CreateData, self.nx, self.ny, rec_nz, /FORCE
        cfg = self.imgFile->GetFileCFG()
        cfg->SetFloat, 'OBS.LAMBDA', lambda
        cfg->SetFloat, 'OBS.DLAMBDA', dlambda
    ENDIF
    ; get and create the optional image file
    nFiles = self.logFT->GetNFiles()
    IF nFiles EQ 1 THEN BEGIN
        self.logFile = self.logFT->GetFile(0)
    ENDIF
ENDIF ELSE BEGIN
    ; here is only the code for algorithms with multiple-value parameters
    ; we expect that the method ExecuteSingleShot has set the
    ; recFile, imgFile, and logFile
    self.recFile->CreateData, self.nx, self.ny, rec_nz, /FORCE
    cfg = self.recFile->GetFileCFG()
    cfg->SetFloat, 'OBS.LAMBDA', lambda
    cfg->SetFloat, 'OBS.DLAMBDA', dlambda
    IF OBJ_VALID(self.imgFile) THEN BEGIN
        ;self.imgFile->CreateData, self.nx, self.ny, 1, /FORCE
        self.imgFile->CreateData, self.nx, self.ny, rec_nz, /FORCE
        cfg = self.imgFile->GetFileCFG()
        cfg->SetFloat, 'OBS.LAMBDA', lambda
        cfg->SetFloat, 'OBS.DLAMBDA', dlambda
    ENDIF
ENDELSE
;self.img = self.imgFile->GetData()
file = self.recFT->GetFile(0)

; get the reconstructed image from the accompanying file
IF self.primresult EQ 0 OR self.primresult EQ 2 THEN BEGIN
   self.rec = file->GetData()
   self.img = self.imgFile->GetData()
ENDIF ELSE BEGIN
    self.imgCube = self.imgFile->GetData()
    self.img = PTR_NEW(FLTARR(self.nx, self.ny))
    self.recCube = file->GetData()
    self.rec = PTR_NEW(FLTARR(self.nx, self.ny))
ENDELSE
self->Load
self.iIter = 0
RETURN, 1
END

FUNCTION ldrsDECONVOLVE::ExecuteStep
RETURN, 1
END

FUNCTION ldrsDECONVOLVE::Get_distances
   RETURN, PTR_NEW()
END

PRO ldrsDECONVOLVE::FindBestReconstruction
IF NOT OBJ_VALID(self.rpsfFile) THEN RETURN
IF NOT OBJ_VALID(self.rimgFile) THEN RETURN
; convolve the reconstruction with the reference psf
result = self.rpsfFile->LoadData()
tmpRec = FFT(*self.rec)*FFT(*(self.rpsfFile->GetData()))
tmpRec = ABS(FFT(tmpRec, /DOUBLE, /INVERSE))
tmpRec = SHIFT(tmpRec, self.nx/2, self.ny/2)/TOTAL(tmpRec)
tmpRef = (*(self.rimgFile->GetData()))/TOTAL(*(self.rimgFile->GetData()))
; recenter reference image and reconstructed image by
; shifting the center of the reconstruction to the center of the reference
;s = SIZE(tmpRec, /DIMENSIONS)
;w = s[0]
;h = s[1]
;ipfft = FFT(tmpRec)
;hcx = ipfft[1,0]/ABS(ipfft[1,0])*COMPLEX(-1.0, 0.0)
;hcy = ipfft[0,1]/ABS(ipfft[0,1])*COMPLEX(-1.0, 0.0)
;xshiftRec = 0.5*ATAN(IMAGINARY(hcx), REAL_PART(hcx))/!PI*w;
;yshiftRec = 0.5*ATAN(IMAGINARY(hcy), REAL_PART(hcy))/!PI*h;
;ipfft = FFT(tmpRef)
;hcx = ipfft[1,0]/ABS(ipfft[1,0])*COMPLEX(-1.0, 0.0)
;hcy = ipfft[0,1]/ABS(ipfft[0,1])*COMPLEX(-1.0, 0.0)
;xshiftRef = 0.5*ATAN(IMAGINARY(hcx), REAL_PART(hcx))/!PI*w;
;yshiftRef = 0.5*ATAN(IMAGINARY(hcy), REAL_PART(hcy))/!PI*h;
;xshift = xshiftRef - xshiftRec
;yshift = yshiftRef - yshiftRec
;LDRS_TRANSLATE_FFT, tmpRec, tmpRec, TX=-xshift, TY=-yshift
; calculate the error
errVal = SQRT(TOTAL((tmpRec - tmpRef)^2)/TOTAL(tmpRef^2))
PRINT, '  ERROR: ', self.iIter, ' ', errVal ;, ' shifts=', xshift, yshift
IF errVal LE self.lastErrMeasure THEN BEGIN
    self.lastErrMeasure = errVal
    *self.lastOpt = *self.rec
ENDIF ELSE BEGIN
    IF (errVal - self.lastErrMeasure)/self.lastErrMeasure GT 0.02 THEN BEGIN
        ;self.breakFlag = 1
        ;PRINT, 'reconstruction (potentially) terminated after ', self.iIter, ' iterations'
    ENDIF
ENDELSE
END

FUNCTION ldrsDECONVOLVE::Execute
PRINT, 'ldrsDECONVOLVE::Execute: self.nIter=', self.nIter, ', self.pIter=', self.pIter
IF self.iIter EQ 0 THEN BEGIN
    IF NOT self->ldrsALGOIMPL::Execute() THEN RETURN, 0
ENDIF
self.breakFlag = 0
WHILE (self.iIter LT self.nIter) AND NOT self.breakFlag DO BEGIN
    self.iIter++
    result = self->ExecuteStep()
    IF (self.iIter MOD self.pIter) EQ 0 THEN BEGIN
        IF NOT OBJ_VALID(self.rimgFile) THEN PRINT, '  iIter=', self.iIter
        LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.rec, NAME='Reconstruction'
        IF self.primresult EQ 1 THEN BEGIN
            (*self.recCube)[*,*,self.iIter/self.pIter - 1] = (*self.rec)[*,*]
            ; convolve the reconstruction with the
            ; reference psf if a image cube is requested
            IF OBJ_VALID(self.rpsfFile) AND OBJ_VALID(self.imgFile) THEN BEGIN
               result = self->create_img(self.rec, self.img)
               (*(self.imgCube))[*,*,self.iIter/self.pIter - 1] = (*self.img)[*,*]
            ENDIF
        ENDIF
        IF (self.primresult EQ 0) OR (self.primresult EQ 1) THEN BEGIN
           self->FindBestReconstruction
        END
    ENDIF
    IF self.pauseFlag THEN RETURN, 0
ENDWHILE
RETURN, 1
END


FUNCTION ldrsDECONVOLVE::create_img, prec, pimg
;creates the image from the reconstruction by convolving the reconstruction
; with the psf
   result = self.rpsfFile->LoadData()
   IF result EQ 0 THEN RETURN, 1
   tmpArr = FFT(*prec)*FFT(*(self.rpsfFile->GetData()))
   tmpArr = ABS(FFT(tmpArr, /DOUBLE, /INVERSE))
   tmpArr = SHIFT(tmpArr, self.nx/2, self.ny/2)
   tot = TOTAL(*(self.rpsfFile->GetData()))
   IF tot NE 0.0 THEN tmpArr *= self.nPhot/tot
   (*pimg)[*,*] = tmpArr[*,*]
   RETURN, 0
END

FUNCTION ldrsDECONVOLVE::Finish
;PRINT, 'ldrsDECONVOLVE::Finish'
IF NOT self->ldrsALGOIMPL::Finish() THEN RETURN, 0
;If the result type is not 0 (lastimage) or 1 (cubeimage) then 
;the reconstruction is stored in the child classes
IF ((self.primresult NE 0) AND (self.primresult NE 1)) THEN RETURN, 1

; store the reconstruction
IF OBJ_VALID(self.rimgFile) AND (self.primresult EQ 0) THEN BEGIN
   PRINT, 'take lastOpt'
   *self.rec = *self.lastOpt
ENDIF
IF self.recFile->StoreData() EQ 0 THEN RETURN, 0
; create the convolved image
IF OBJ_VALID(self.rpsfFile) AND OBJ_VALID(self.imgFile) THEN BEGIN
    IF self.primresult EQ 0 THEN BEGIN
        result = self->create_img(self.rec, self.img)
    ENDIF

    result = self.imgFile->StoreData()
    LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.imgFile->GetData(), NAME='Reconstructed Image'
ENDIF
self.iIter = 0
RETURN, 1
END

; the number of parameters which can have multiple values which are
; used successively
FUNCTION ldrsDECONVOLVE::GetNoMP
RETURN, 0
END

; short tag which is used as a prefix in the generated file name
FUNCTION ldrsDECONVOLVE::GetMPTag, nr
RETURN, ''
END

; label whch is used in the user interface
FUNCTION ldrsDECONVOLVE::GetMPLabel, nr
RETURN, ''
END

; get a string which contains all values of a specific parameter
FUNCTION ldrsDECONVOLVE::GetMPValues, nr
RETURN, ''
END

; set a parameter to a specific value which is givan as a string
PRO ldrsDECONVOLVE::SetMP, nr, val
END

PRO ldrsDECONVOLVE::CreateAllResultFiles, n, m, vals, counts
COMMON ldrs, env
recPre = self.recFT->GetPrefix()
imgPre = self.imgFT->GetPrefix()
logPre = self.logFT->GetPrefix()
recExt = self.recFT->GetExtension()
imgExt = self.imgFT->GetExtension()
logExt = self.logFT->GetExtension()
FOR i=0, n-1 DO BEGIN
    ; create the file name part containing the values
    fnpart = ''
    FOR j=m-1, 0, -1 DO BEGIN
        k = i MOD counts[j]
        fnpart = '_' + self->GetMPTag(j) + (*vals[j])[k] + fnpart
    ENDFOR
    IF recPre NE '' THEN BEGIN
        fname = recPre + fnpart + '.' + recExt
        file = self.recFT->GetFile(i)
        IF NOT OBJ_VALID(file) THEN file = self.recFT->CreateFile(fname)
        file->SetName, fname
    ENDIF
    IF imgPre NE '' THEN BEGIN
        fname = imgPre + fnpart + '.' + imgExt
        file = self.imgFT->GetFile(i)
        IF NOT OBJ_VALID(file) THEN file = self.imgFT->CreateFile(fname)
        file->SetName, fname
    ENDIF
    IF logPre NE '' THEN BEGIN
        fname = logPre + fnpart + '.' + logExt
        file = self.logFT->GetFile(i)
        IF NOT OBJ_VALID(file) THEN file = self.logFT->CreateFile(fname)
        file->SetName, fname
    ENDIF
ENDFOR
IF recPre NE '' THEN BEGIN
    env.recipe->Broadcast, 'FiletypeChanged', self.recFT
ENDIF
IF imgPre NE '' THEN BEGIN
    env.recipe->Broadcast, 'FiletypeChanged', self.imgFT
ENDIF
IF logPre NE '' THEN BEGIN
    env.recipe->Broadcast, 'FiletypeChanged', self.logFT
ENDIF
END

PRO ldrsDECONVOLVE::ExecuteSingleShot, n, vals, counts, i
IF i LT 0 THEN RETURN
; switch to the files for that sigle shot
self.recFile = self.recFT->GetFile(i)
self.imgFile = self.imgFT->GetFile(i)
self.logFile = self.logFT->GetFile(i)
; open the current log file
LDRS_LOG_CLEAR
IF OBJ_VALID(self.logFile) THEN BEGIN
    lfn = self.logFile->GetName()
    LDRS_LOG_APPEND, '### Starting logging into file ' + lfn
    LDRS_LOG_OPEN, lfn
ENDIF
LDRS_LOG_APPEND, '#############'
LDRS_LOG_APPEND, STRING(FORMAT='("### Performing a deconvolution, ", I0, " of ", I0, " alternatives, ", I0, " variable parameters:")', i+1, PRODUCT(counts), n)
; set the parameter to the requested values
FOR j=n-1, 0, -1 DO BEGIN
    k = i MOD counts[j]
    LDRS_LOG_APPEND, STRING('    ', self->GetMPLabel(j), ' = ', (*vals[j])[k])
    self->SetMP, j, (*vals[j])[k]
ENDFOR
self->ldrsALGOIMPL::ExecuteAlgorithm
LDRS_LOG_APPEND, '### finished'
; close the current log file
LDRS_LOG_CLOSE
END

PRO ldrsDECONVOLVE::ExecuteAlgorithm, ALL=all, SINGLE=single
; check if the algorithm can use multiple parameters
nMP = self->GetNoMP()
IF nMP EQ 0 THEN BEGIN
    ; no multiple parameters -> normal execution
    self->ldrsALGOIMPL::ExecuteAlgorithm
    RETURN
END
; use an array of pointers to arrays of strings which holds all
; parameter values as single strings
self->Load
mpVals = PTRARR(nMP)
mpCount = INTARR(nMP)
nRuns = 1
; get all values, splits them and store them
FOR i=0, nMP-1 DO BEGIN
    mpVals[i] = PTR_NEW(STRSPLIT(self->GetMPValues(i), ';', /EXTRACT))
    mpCount[i] = N_ELEMENTS(*mpVals[i])
    nRuns *= mpCount[i]
ENDFOR
self->CreateAllResultFiles, nRuns, nMP, mpVals, mpCount
IF N_ELEMENTS(all) NE 0 THEN BEGIN
    FOR i=0, nRuns-1 DO BEGIN
        IF self.pauseFlag THEN BREAK
        self->ExecuteSingleShot, nMP, mpVals, mpCount, i
    ENDFOR
    RETURN
ENDIF
IF N_ELEMENTS(single) NE 0 THEN BEGIN
    IF SIZE(single, /DIMENSIONS) EQ 0 THEN BEGIN
        self->ExecuteSingleShot, nMP, mpVals, mpCount, single
        RETURN
    ENDIF
    FOR i=0, N_ELEMENTS(list)-1 DO BEGIN
        IF self.pauseFlag THEN BREAK
        self->ExecuteSingleShot, nMP, mpVals, mpCount, list[i]
    ENDFOR
    RETURN
ENDIF
END

PRO ldrsDECONVOLVE::Load, cfg
self->ldrsALGOIMPL::Load, cfg
; new code: use the module configuration
hcfg = cfg
; old code: use the file configuration
;hcfg = self->Locate(self.recFT, /NO_CREATE)
IF OBJ_VALID(hcfg) THEN BEGIN
    self.primresult = hcfg->GetInt('PRIMRESULT', DEFAULT=0)
    self.secresult = hcfg->GetInt('SECRESULT', DEFAULT=0)
    self.primdist_selected = hcfg->GetInt('PRIMDIST', DEFAULT=0)
    self.secdist_selected = hcfg->GetInt('SECDIST', DEFAULT=0)
    self.nIter = hcfg->GetInt('NITER', DEFAULT=1000)
    self.pIter = hcfg->GetInt('PITER', DEFAULT=10)
ENDIF ELSE BEGIN
    self.nIter = 1000
    self.pIter = 10
    self.primresult = 1
    self.secresult = 0
    self.primdist_selected = 0
    self.secdist_selected  = 0
ENDELSE
END

PRO ldrsDECONVOLVE::Store, cfg
self->ldrsALGOIMPL::Store, cfg
; new code: use the module configuration
hcfg = cfg
; old code: use the file configuration
;hcfg = self->Locate(self.recFT)
IF OBJ_VALID(hcfg) THEN BEGIN
    hcfg->SetInt, 'NITER', self.nIter
    hcfg->SetInt, 'PITER', self.pIter
    hcfg->SetInt, 'PRIMRESULT',  self.primresult
    hcfg->SetInt, 'SECRESULT',  self.secresult
    hcfg->SetInt, 'PRIMDIST',  self.primdist_selected
    hcfg->SetInt, 'SECDIST',  self.secdist_selected
ENDIF
END

PRO ldrsDECONVOLVE::ReadDynamicContent
WIDGET_CONTROL, self.ddNIter, SET_VALUE=STRING(self.nIter)
WIDGET_CONTROL, self.ddPIter, SET_VALUE=STRING(self.pIter)
WIDGET_CONTROL, self.ddPrimResult, SET_COMBOBOX_SELECT=self.primresult
WIDGET_CONTROL, self.ddSecResult, SET_COMBOBOX_SELECT=self.secresult
WIDGET_CONTROL, self.ddDistNames, SET_COMBOBOX_SELECT=self.primdist_selected
WIDGET_CONTROL, self.ddDistNames2, SET_COMBOBOX_SELECT=self.secdist_selected
END

PRO ldrsDECONVOLVE::ItemSelected, id, item
IF id EQ self.ddPrimResult THEN self.primresult = item
IF id EQ self.ddDistNames THEN self.primdist_selected = item
IF id EQ self.ddDistNames2 THEN self.secdist_selected = item
IF id EQ self.ddSecResult THEN self.secresult = item
END



PRO ldrsDECONVOLVE::WriteDynamicContent
WIDGET_CONTROL, self.ddNIter, GET_VALUE=str
self.nIter = FIX(str)
WIDGET_CONTROL, self.ddPIter, GET_VALUE=str
self.pIter = FIX(str)
END

PRO ldrsDECONVOLVE::CreateContent
self->ldrsALGOIMPL::CreateContent
row = self->GetFreeRow()
lab            = self->CreateLabel(0, row, 'Iterations:', /HALIGN_RIGHT)
self.ddNIter   = self->CreateEntry(2, row, VALUE='10000')
row++

lab            = self->CreateLabel(0, row, 'Show every:', /HALIGN_RIGHT)
self.ddPIter   = self->CreateEntry(2, row, VALUE='100')
row++

lab            = self->CreateLabel(0, row, 'Result:', /HALIGN_RIGHT)

lab            = self->CreateLabel(1, row, 'Primary:', /HALIGN_RIGHT)
IF (PTR_VALID(self->Get_distances())) THEN BEGIN
   self.ddPrimResult  = self->CreateComboBox(2, row, ['Last', 'Cube', 'Min single Dist', 'Min all Dist'])
   lab            = self->CreateLabel(3, row, 'Dist:', /HALIGN_RIGHT)
   self.ddDistNames  = self->CreateComboBox(4, row, (*(self->Get_distances())))   
   row++
   lab            = self->CreateLabel(1, row, 'Secondary:', /HALIGN_RIGHT)
   self.ddSecResult  = self->CreateComboBox(2, row, [ 'None', 'Min single Dist', 'Min all Dist'])
   lab            = self->CreateLabel(3, row, 'Dist:', /HALIGN_RIGHT)
   self.ddDistNames2  = self->CreateComboBox(4, row, (*(self->Get_distances())))   
ENDIF ELSE BEGIN
   self.ddPrimResult  = self->CreateComboBox(2, row, ['Last', 'Cube'])
   lab            = self->CreateLabel(3, row, 'Dist:', /HALIGN_RIGHT)
   self.ddDistNames  = self->CreateComboBox(4, row, 'Not available')   
   row++
   lab            = self->CreateLabel(1, row, 'Secondary:', /HALIGN_RIGHT)
   self.ddSECResult  = self->CreateComboBox(2, row, ['Last', 'Cube'])
   lab            = self->CreateLabel(3, row, 'Dist:', /HALIGN_RIGHT)
   self.ddDistNames2  = self->CreateComboBox(4, row, 'Not available')   
ENDELSE
row++

;self->UpdateGui
END

PRO ldrsDECONVOLVE__define
struct = { ldrsDECONVOLVE, $
           INHERITS ldrsALGOIMPL, $
           objFT:OBJ_NEW(), $
           psfFT:OBJ_NEW(), $
           rpsfFT:OBJ_NEW(), $
           rimgFT:OBJ_NEW(), $
           recFT:OBJ_NEW(), $
           imgFT:OBJ_NEW(), $
           logFT:OBJ_NEW(), $
           rpsfFile:OBJ_NEW(), $
           rimgFile:OBJ_NEW(), $
           recFile:OBJ_NEW(), $
           imgFile:OBJ_NEW(), $
           logFile:OBJ_NEW(), $
           ddNIter:0, ddPIter:0, ddPrimResult:0, ddDistNames:0, ddDistNames2:0, ddSecResult:0, $
           primresult:1, $
           secresult:0, $
           nphot:0.0, $
           nIter:0, pIter:0, cube:0, iIter:0, $
           nx:0, ny:0, nPA:0, $
           rec:PTR_NEW(), $
           recCube:PTR_NEW(), $
           img:PTR_NEW(), $
           imgCube:PTR_NEW(), $
           lastErrMeasure:0.0, $
           lastOpt:PTR_NEW(), $
           breakFlag:0, $
           primdist_selected:0, $
           secdist_selected:0 $
         }
END
