FUNCTION ldrsEXTRACTION::Init, algorithm
  IF NOT self->ldrsALGOIMPL::Init(algorithm) THEN RETURN, 0
  self.dlgTitle = 'Extraction Setup'
; set the default parameters
  self.nResults = 2
  self.removeBgrd = 0
  self.useMask = 0
  self.dstAutoDim = 1           ;
  self.dstNX = 0
  self.dstNY = 0
  RETURN, 1
END

PRO ldrsEXTRACTION::CalcDimensions, srcFT, cfgName, nx, ny, nz
  PRINT, 'calculate the bounding box for all cut out objects'
; (beware of the meaning of margins!)
  leftMax = 0
  rightMax = 0
  topMax = 0
  bottomMax = 0
  nz = 0
  self.dstCX = 0.0
  self.dstCY = 0.0
  self.srcCount = 0
  self.srcNr = 0
  FOR i=0,N_ELEMENTS(self.dstSrcCounts)-1 DO self.dstSrcCounts[i] = 0
  PRINT, '  iterate over all input files'
  nFiles = srcFT->GetNFiles()
  IF self.nResults EQ 0 THEN nz = 1
  IF self.nResults EQ 1 THEN nz = nFiles
  FOR iFile=0,nFiles-1 DO BEGIN
     srcFile = srcFT->GetFile(iFile)
     PRINT, '    processing file number ', iFile, ' => name=', srcFile->GetName()
     IF NOT srcFile->CheckContent(self.content) THEN CONTINUE
     IF srcFile->LoadData() EQ 0 THEN RETURN
     PRINT, '      iterate over all images in this file'
     nImages = srcFile->GetNZ()
     IF self.nResults EQ 2 THEN nz += nImages
     nObjs = 0
     FOR iImage=0,nImages-1 DO BEGIN
        cfg = srcFile->GetImageCFG(iImage)
        IF NOT OBJ_VALID(cfg) THEN BEGIN
           PRINT, '        no valid image cfg for image ', iImage, ' found'
           IF self.nResults EQ 3 THEN nz += nObjs
           self.srcCount += nObjs
           CONTINUE
        ENDIF
        hcfg = cfg->LocateByName(cfgName, /NO_CREATE)
        IF NOT OBJ_VALID(hcfg) THEN BEGIN
           PRINT, '        no valid object cfg for ', cfgName, ' in image cfg found'
           IF self.nResults EQ 3 THEN nz += nObjs
           self.srcCount += nObjs
           CONTINUE
        ENDIF
        PRINT, '        iterate over all objects in this image'
        nObjs = hcfg->GetNChilds()
        IF self.nResults EQ 3 THEN nz += nObjs
        self.srcCount += nObjs
        FOR iObj=1, nObjs DO BEGIN
           hhcfg = hcfg->LocateByNr(iObj)
           IF NOT OBJ_VALID(hhcfg) THEN CONTINUE
           hhcfg->GetFloat4D, 'BOX', hl, hr, ht, hb
           IF hl GT leftMax THEN leftMax = hl
           IF hr GT rightMax THEN rightMax = hr
           IF ht GT topMax THEN topMax = ht
           IF hb GT bottomMax THEN bottomMax = hb
        ENDFOR
     ENDFOR
  ENDFOR
  nx = leftMax + rightMax
  ny = bottomMax + topMax
  self.dstCount = nz
  IF self.dstAutoDim EQ 1 THEN BEGIN
     self.dstNX = nx
     self.dstNY = ny
     self.dstCX = leftMax
     self.dstCY = bottomMax
  ENDIF
  PRINT, '  obj dimensions ', nx, ny, nz, ', dstCX,dstCY ', self.dstCX, self.dstCY
END

PRO ldrsEXTRACTION::ProcSky, part, partCFG
  proc = LDRS_GET_PROCEDURE('SKYC', partCFG->GetInt('SKYALGO', DEFAULT=0))
  CALL_PROCEDURE, proc, part, partCFG, self.removeBgrd, hbgrd
  self.bgrd += hbgrd
;PRINT, '  ProcOBJ: total after=', TOTAL(psf)
END

PRO ldrsEXTRACTION::ProcHalo, part, partCFG, haloCFG, intData, iImage
  partCFG->GetFloat2D, 'POS', x, y
  partCFG->GetFloat4D, 'BOX', left, right, top, bottom
  sl = FIX(x - left + 0.5)
  sr = FIX(x + right + 0.5) - 1
  sb = FIX(y - bottom + 0.5)
  st = FIX(y + top + 0.5) - 1
  proc = LDRS_GET_PROCEDURE('HALOC', partCFG->GetInt('HALOALGO', DEFAULT=0))
  CALL_PROCEDURE, proc, part, sl, sr, sb, st, haloCFG, intData, iImage
END

PRO ldrsEXTRACTION::ProcExtrapolation, part, dl, dr, db, dt, partCFG
  nx = dr - dl + 1
  ny = dt - db + 1
  IF (nx EQ self.dstNX) AND (ny EQ self.dstNY) THEN RETURN
  proc = LDRS_GET_PROCEDURE('PSFEXT', partCFG->GetInt('EPOLALGO', DEFAULT=0))
  CALL_PROCEDURE, proc, part, self.dstNX, self.dstNY, dl, dr, db, dt
END

PRO ldrsEXTRACTION::ProcImage, srcData, iImage, partCFG, haloCFG, partData, nx, ny, nz, z, pa, lambda, dlambda
  PRINT, '  ProcImage:', iImage, nx, ny, nz, z, pa, lambda
  nParts = partCFG->GetNChilds()
  IF nParts EQ 0 THEN RETURN
;PRINT, 'ProcImage: total before=', total((*partData)[*, *, z])
  spart = FLTARR(self.dstNX, self.dstNY)
  FOR iPart=1, nParts DO BEGIN
     hcfg = partCFG->LocateByNr(iPart)
     IF NOT OBJ_VALID(hcfg) THEN CONTINUE
                                ; Extract the object (target or calibrator)
                                ; and apply sky subtraction and/or
                                ; halo compensation
     hcfg->GetFloat2D, 'POS', x, y
     hcfg->GetFloat4D, 'BOX', left, right, top, bottom
     sl = FIX(x - left + 0.5)
     sr = FIX(x + right + 0.5) - 1
     sb = FIX(y - bottom + 0.5)
     st = FIX(y + top + 0.5) - 1
     part = (*srcData)[sl : sr, sb : st, iImage]
     self->ProcSky, part, hcfg
     self->ProcHalo, part, hcfg, haloCFG, srcData, iImage
                                ; place the object into the final array
                                ; and extrapolate the missing pixel values
     dl = FIX(self.dstCX - left)
     dr = FIX(self.dstCX + right - 1)
     db = FIX(self.dstCY - bottom)
     dt = FIX(self.dstCY + top - 1)
     PRINT, '    s = ', sl, sr, sb, st, ', d = ', dl, dr, db, dt
     spart[*] = 0.0
     spart[dl:dr, db:dt] = part
     self->ProcExtrapolation, spart, dl, dr, db, dt, partCFG
     IF self.useMask EQ 0 THEN BEGIN
        (*partData)[*, *, z] += spart
     ENDIF ELSE BEGIN
        LDRS_CREATE_MASK, nx, ny, pa/!RADEG, lambda, dlambda, mask, /CLEAN
        mask = SHIFT(mask, nx/2, ny/2)
        IF self.recenter THEN BEGIN
           partimage = REAL_PART(FFT(FFT(spart, -1)*mask, 1))
           LDRS_TRANSLATE_FFT, partimage, npartimage
           (*partData)[*, *, z] += npartimage
        ENDIF ELSE BEGIN
           (*partData)[*, *, z] += REAL_PART(FFT(FFT(spart, -1)*mask, 1))
        ENDELSE
     ENDELSE
                                ; store the current source parameters, switch to the next source,
                                ; and increment the number of sources for the current destination
     self.srcPA[self.srcNr] = pa
     self.srcLambda[self.srcNr] = lambda
     self.srcDLambda[self.srcNr] = dlambda
     self.srcNr++
     self.dstSrcCounts[z]++
     IF self.nResults EQ 3 THEN z++
  ENDFOR
;PRINT, 'ProcImage: total after=', total((*partData)[*, *, z])
END

PRO ldrsEXTRACTION::ProcFile, srcFile, cfgName, dstFile, partData, nx, ny, nz, z
  PRINT, '  ProcFile: ', srcFile->GetName(), ', ', cfgName, ', ', dstFile->GetName()
; iterate over all images in this file
  srcData = srcFile->GetData()
  nImages = srcFile->GetNZ()
  cfg = srcFile->GetFileCFG()
  partCFG = OBJ_NEW()
  haloCFG = OBJ_NEW()
  hcfg = cfg->Locate('OBS.LAMBDA', /NO_CREATE)
  IF OBJ_VALID(hcfg) THEN BEGIN
     lambda = hcfg->GetFloat('', DEFAULT=2.2)
     hcfg = dstFile->GetFileCFG()
     hcfg->SetFloat, 'OBS.LAMBDA', lambda
  ENDIF ELSE BEGIN
     lambda = 2.2
  ENDELSE
  hcfg = cfg->Locate('OBS.DLAMBDA', /NO_CREATE)
  IF OBJ_VALID(hcfg) THEN BEGIN
     dlambda = hcfg->GetFloat('', DEFAULT=0.0)
     hcfg = dstFile->GetFileCFG()
     hcfg->SetFloat, 'OBS.DLAMBDA', dlambda
  ENDIF ELSE BEGIN
     dlambda = 0.0
  ENDELSE
  pa = 0.0
; loop over all interferograms in order to build the part image
  FOR iImage=0,nImages-1 DO BEGIN
                                ; check if this image has a part star specification
                                ; if yes => use the new one
     cfg = srcFile->GetImageCFG(iImage)
     IF OBJ_VALID(cfg) THEN BEGIN
        rotator = cfg->GetFloat('OBS.ROTATOR', DEFAULT=0.0)
        rotate  = cfg->GetFloat('PREP.ROTATE', DEFAULT=0.0)
        pa = rotator + rotate
        hcfg = cfg->LocateByName(cfgName, /NO_CREATE)
        IF OBJ_VALID(hcfg) THEN partCFG = hcfg
        hcfg = cfg->LocateByName('HALO', /NO_CREATE)
        IF OBJ_VALID(hcfg) THEN haloCFG = hcfg
     ENDIF
     IF NOT OBJ_VALID(partCFG) THEN CONTINUE ; no valid part star =< continue
     self->ProcImage, srcData, iImage, partCFG, haloCFG, partData, nx, ny, nz, z, pa, lambda, dlambda
     IF self.nResults EQ 2 THEN BEGIN
        hhcfg = dstFile->GetImageCFG(z)
        hhcfg->SetFloat, 'BGRD', self.bgrd
        hhcfg->SetFloat, 'ANGLE', pa
        self.bgrd = 0.0
        z++
     ENDIF
  ENDFOR
; show the ceated part
  LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=partData, NAME='PART'
  WAIT, 2
  RETURN
END

FUNCTION ldrsEXTRACTION::ExtractPart, srcFT, cfgName, dstFT
  PRINT, 'ldrsEXTRACTION::ExtractPart:'
; calculate the dimensions of the result
  self->CalcDimensions, srcFT, cfgName, nx, ny, nz
  IF (nx EQ 0 ) OR (ny EQ 0) OR (nz EQ 0) THEN RETURN, 0
  self.bgrd = 0.0
  self.srcNr = 0
; create the part image (nx, ny, nz)
  dstFile = dstFT->GetFile(0)
  dstFile->CreateData, self.dstNX, self.dstNY, nz, /FORCE
  partData = dstFile->GetData()
  IF NOT PTR_VALID(partData) THEN RETURN, 0
  (*partData)[*] = 0.0
; loop over all files
  nFiles = srcFT->GetNFiles()
  z = 0
  FOR iFile=0,nFiles-1 DO BEGIN
     srcFile = srcFT->GetFile(iFile)
     IF NOT srcFile->CheckContent(self.content) THEN CONTINUE
     self->ProcFile, srcFile, cfgName, dstFile, partData, nx, ny, nz, z
     IF self.nResults EQ 1 THEN  BEGIN
        hhcfg = dstFile->GetImageCFG(z)
        hhcfg->SetFloat, 'BGRD', self.bgrd
        self.bgrd = 0.0
        z++
     ENDIF
  ENDFOR
  IF self.nResults EQ 0 THEN  BEGIN
     hhcfg = dstFile->GetImageCFG(z)
     hhcfg->SetFloat, 'BGRD', self.bgrd
     self.bgrd = 0.0
  ENDIF
  RETURN, 1
END

FUNCTION ldrsEXTRACTION::Prepare
  PRINT, 'ldrsEXTRACTION::Prepare'
  IF NOT self->ldrsALGOIMPL::Prepare() THEN RETURN, 0
  RETURN, 1
END

PRO ldrsEXTRACTION__define
  struct = { ldrsEXTRACTION, $
             INHERITS ldrsALGOIMPL, $
             content:3, $
             nResults:0, removeBgrd:0, useMask:0, recenter:0, $
             srcCount:0, srcNr:0, srcPA:FLTARR(100), srcLambda:FLTARR(100), srcDLambda:FLTARR(100), $
             dstCount:0, dstSrcCounts:INTARR(100), $
             dstAutoDim:0, dstNX:0, dstNY:0, dstCX:0, dstCY:0, $
             bgrd:0.0 $
           }
END
