;##########################
;# File
;FILE<i>.TAG     = <string>           # unique file tag
;FILE<i>.LABEL   = <string>           # label used in the GUI
;FILE<i>.COLOR   = "<r> <g> <b>"      # color used in the flowchart
;FILE<i>.BOX     = "<x> <y> <w> <h>"  # rectangle in the flowchart
;FILE<i>.MINCARD = <int>              # minimum number of files
;FILE<i>.MAXCARD = <int>              # maximum number of files
FUNCTION ldrsFILETYPE::Init, recipe, cfg
IF NOT self->ldrsNODE::Init(recipe, cfg) THEN RETURN, 0
self.minCard = cfg->GetInt('MINCARD')
self.maxCard = cfg->GetInt('MAXCARD')
self.fileClass = cfg->GetString('CLASS')
self.extension = cfg->GetString('EXTENSION')
self.hasPrefix = 0
self.prefix = ''
;PRINT, "ldrsFILETYPE::Init, ", self.tag
RETURN, 1
END

FUNCTION ldrsFILETYPE::GetMinCard
RETURN, self.minCard
END

FUNCTION ldrsFILETYPE::GetMaxCard
RETURN, self.maxCard
END

FUNCTION ldrsFILETYPE::GetExtension
RETURN, self.extension
END

FUNCTION ldrsFILETYPE::GetPrefix
IF self.hasPrefix THEN RETURN, self.prefix ELSE RETURN, ''
END

PRO ldrsFILETYPE::SetPrefix, t
self.prefix = t
recName = self.recipe->GetName()
ht = LDRS_ENCODE_FILENAME(recName, self.prefix)
IF ht EQ '' THEN self.hasPrefix = 0 ELSE self.hasPrefix = 1
END

FUNCTION ldrsFILETYPE::GetNFiles
RETURN, self.nFiles
END

FUNCTION ldrsFILETYPE::GetFile, nr
IF nr GE self.nFiles THEN RETURN, OBJ_NEW()
RETURN, self.files[nr]
END

FUNCTION ldrsFILETYPE::CheckCardinality, minCard,  maxCard
; a minimum of minCard filenames are needed
IF self.nFiles LT minCard THEN BEGIN
    PRINT, '  ', minCard - self.nFiles, ' filename(s) missing'
    RETURN, 0
ENDIF
; not more than maxCard filenames are needed
IF self.nFiles GT maxCard THEN BEGIN
    PRINT, '  ', self.nFiles - maxCard, ' too many filename(s)'
    RETURN, 0
ENDIF
; check if every file has a filename
FOR i=0, self.nFiles-1 DO BEGIN
    file = self.files[i]
    name = file->GetName()
    ; check if a valid filename is given
    IF name EQ '' THEN BEGIN
        PRINT, '  no filename for ', i+1, 'th file given'
        RETURN, 0
    ENDIF
END
RETURN, 1
END

FUNCTION ldrsFILETYPE::CheckExistance
; check if every filename does exist
FOR i=0, self.nFiles-1 DO BEGIN
    file = self.files[i]
    name = file->GetName()
    ; check if a valid filename is given
    IF name EQ '' THEN BEGIN
        PRINT, '  no filename for ', i+1, 'th file given'
        RETURN, 0
    ENDIF
    ; check if the file exists and is a regular file
    fi = FILE_INFO(name)
    IF fi.regular NE 1 THEN BEGIN
        PRINT, '  file ', name, ' does not exist or is not a regular file'
        RETURN, 0
    ENDIF
    ;PRINT, '  file ', name, ' does exist and is a regular file'
END
RETURN, 1
END

FUNCTION ldrsFILETYPE::CreateFile, name
IF self.nFiles GE self.maxCard THEN RETURN, OBJ_NEW()
file = OBJ_NEW(self.fileClass, self)
file->SetName, name
self.files[self.nFiles] = file
self.nFiles += 1

RETURN, file
END

PRO ldrsFILETYPE::RemoveFile, file
nr = -1
FOR i=0, self.nFiles - 1 DO BEGIN
    IF self.files[i] EQ file THEN BEGIN
        nr = i
        BREAK
    ENDIF
ENDFOR
IF nr EQ -1 THEN RETURN
OBJ_DESTROY, file
FOR i = nr, self.nFiles - 2 DO self.files[i] = self.files[i + 1]
self.files[self.nFiles - 1] = OBJ_NEW()
self.nFiles -= 1
END

PRO ldrsFILETYPE::Store, cfg
hcfg = cfg->GetParent()
hcfg = hcfg->Locate('PREFIX')
recName = self.recipe->GetName()
hcfg->SetString, self->GetTag(), LDRS_ENCODE_FILENAME(recName, self.prefix)
FOR i = 0, self.nFiles - 1 DO BEGIN
    child = cfg->LocateByNr(i+1)
    self.files[i]->Store, child
ENDFOR
END

PRO ldrsFILETYPE::Load, cfg
hcfg = cfg->GetParent()
hcfg = hcfg->Locate('PREFIX')
self.prefix = hcfg->GetString(self->GetTag(), DEFAULT='')
IF self.prefix EQ '' THEN self.hasPrefix = 0 ELSE self.hasPrefix = 1
recName = self.recipe->GetName()
self.prefix = LDRS_DECODE_FILENAME(recName, self.prefix)
WHILE OBJ_VALID(self.files[0]) DO self->RemoveFile, self.files[0]
child = cfg->GetFirst()
WHILE OBJ_VALID(child) DO BEGIN
    file = self->CreateFile('')
    file->Load, child
    child = child->GetNext()
END
END

PRO ldrsFILETYPE__define
struct = { ldrsFILETYPE, $
           INHERITS ldrsNODE, $
           minCard:0, $
           maxCard:30, $
           fileClass:'', $
           extension:'', $
           hasPrefix:0, $
           prefix:'', $
           nFiles:0, $
           files:OBJARR(32) $
           }
END
