; Die Informationen, die sich auf einzelne Bilder eines
; Cubes beziehen werden so lange wie moeglich behalten
; und erst dann verworfen wenn das Bild entweder geladen
; wurde oder der Nutzer diese Informationen bearbeiten moechte.
FUNCTION ldrsFITSFILE::Init, filetype
IF NOT self->ldrsFILE::Init(filetype) THEN RETURN, 0
RETURN, 1
END

FUNCTION ldrsFITSFILE::GetNImages
RETURN, self.nImages
END

FUNCTION ldrsFITSFILE::GetNX
RETURN, self.nx
END

FUNCTION ldrsFITSFILE::GetNY
RETURN, self.ny
END

FUNCTION ldrsFITSFILE::GetNZ
RETURN, self.nImages
END

FUNCTION ldrsFITSFILE::GetImageCFG, nr
hcfg = self.cfg->LocateByName('IMG')
hhcfg = hcfg->LocateByNr(nr+1)
RETURN, hhcfg
END

;PRO ldrsFILE::SetImageCFG, nr, cfg
;IF NOT self->Resize(NIMAGES=nr+1) THEN RETURN
;(*self.cfgs)[nr] = cfg
;END

FUNCTION ldrsFITSFILE::Resize, nImages=n
; if the keyword is not set we have to get the number of
; images from a fits file
IF KEYWORD_SET(n) THEN BEGIN
    IF n LE self.nImages THEN RETURN, 1
ENDIF ELSE BEGIN
    IF self.name EQ ''THEN RETURN, 0
    ;PRINT, 'ldrsFITSFILE::Resize: testing file ', self.name
    ; check if the file exists and is a regular file
    fi = FILE_INFO(self.name)
    IF fi.regular NE 1 THEN RETURN, 0
    ; read all header keywords from the fits file
    FITS_OPEN, self.name, fcb
    FITS_CLOSE, fcb
    ; check the dimensions of the primary image
    IF fcb.naxis[0] EQ 2 THEN BEGIN
        n = 1
    ENDIF ELSE BEGIN
        IF fcb.naxis[0] EQ 3 THEN BEGIN
            n = fcb.axis[2, 0]
        ENDIF ELSE BEGIN
            PRINT, 'ldrsFITSFILE::Resize, naxis[0] > 3!'
        ENDELSE
    ENDELSE
    ; check if the number of images is now defined
    IF NOT KEYWORD_SET(n) THEN RETURN, 0 ; the keyord is not set and no file available!
    ; check if the number of images is equal to the old value
    IF n EQ self.nImages THEN RETURN, 1 ; do nothing
ENDELSE
; store the new number of images
hn = self.nImages
self.nImages = n
RETURN, 1
END

FUNCTION ldrsFITSFILE::LoadData, FORCE=force
;PRINT, 'ldrsFITSFILE::LoadData, ', self.name
IF NOT KEYWORD_SET(force) THEN force = 0
res = self->Resize()
IF PTR_VALID(self.data) AND NOT force THEN RETURN, 1
fi = FILE_INFO(self.name)
IF fi.regular NE 1 THEN RETURN, 0
FITS_OPEN, self.name, fcb
FITS_READ, fcb, data, EXTEN_NO=0, /NO_ABORT, MESSAGE=msg
FITS_CLOSE, fcb
IF msg NE '' THEN BEGIN
    PRINT, 'ldrsFITSFILE::LoadData, ', self.name, ' errmsg=', msg
    RETURN, 0
ENDIF
PRINT, '   fcb.axis = ', fcb.axis
IF N_ELEMENTS(fcb.naxis) EQ 1 THEN BEGIN
   PRINT, 'Single HDU Image'
   self.nx = fcb.axis[0, 0]
   self.ny = fcb.axis[1, 0]
   IF fcb.naxis EQ 3 THEN self.nImages = fcb.axis[2, 0] ELSE self.nImages = 1
ENDIF ELSE BEGIN
   PRINT, 'Multi HDU Image'
   self.nx = fcb.axis[0, 0, 0]
   self.ny = fcb.axis[1, 0, 0]
   IF fcb.naxis[0] EQ 3 THEN self.nImages = fcb.axis[2, 0, 0] ELSE self.nImages = 1
ENDELSE
IF PTR_VALID(self.data) THEN PTR_FREE, self.data
self.data = PTR_NEW(data)
PRINT, 'ldrsFITSFILE::LoadData, ', self.name, ' => Loaded, nx,ny,nz=', self.nx, self.ny, self.nImages
RETURN, 1
END

FUNCTION ldrsFITSFILE::StoreData
PRINT, 'ldrsFITSFILE::StoreData, ', self.name
IF NOT PTR_VALID(self.data) THEN RETURN, 0
FITS_WRITE, self.name, *self.data, /NO_ABORT, MESSAGE=msg
IF msg NE '' THEN BEGIN
    PRINT, 'ldrsFITSFILE::StoreData, ', self.name, ' errmsg=', msg
    RETURN, 0
ENDIF
RETURN, 1
END

FUNCTION ldrsFITSFILE::StoreDataHeader, header
PRINT, 'ldrsFITSFILE::StoreDataHeader, ', self.name
IF NOT PTR_VALID(self.data) THEN RETURN, 0
FITS_WRITE, self.name, *self.data, header, /NO_ABORT, MESSAGE=msg
IF msg NE '' THEN BEGIN
    PRINT, 'ldrsFITSFILE::StoreDataHeader, ', self.name, ' errmsg=', msg
    RETURN, 0
ENDIF
RETURN, 1
END

FUNCTION ldrsFITSFILE::AppendData, header, pdata
PRINT, 'ldrsFITSFILE::AppendData, ', self.name
IF NOT PTR_VALID(pdata) THEN RETURN, 0
WRITEFITS, self.name, *pdata, header, /APPEND
RETURN, 1
END

PRO ldrsFITSFILE::CreateData, nx, ny, nz, FORCE=force
IF N_ELEMENTS(force) EQ 0 THEN force = 0
IF PTR_VALID(self.data) AND NOT force THEN RETURN
PRINT, 'ldrsFITSFILE::CreateData ', self.name, nx, ny, nz
data = FLTARR(nx, ny, nz)
self.nx = nx
self.ny = ny
self.nImages = nz
IF PTR_VALID(self.data) THEN PTR_FREE, self.data
self.data = PTR_NEW(data)
END

FUNCTION ldrsFITSFILE::GetData
RETURN, self.data
END

PRO ldrsFITSFILE__define
struct = { ldrsFITSFILE, $
           INHERITS ldrsFILE, $
           data:PTR_NEW(), $
           nImages:0, $
           nx:0, $
           ny:0 $
         }
END
