FUNCTION ldrsHALO::Init, tool, nr, font
IF NOT self->ldrsIMGAREA::Init(tool, nr, font) THEN RETURN, 0
;PRINT, 'ldrsHALO::Init, col=', self.col
self.labelFlag = 1
self.crossFlag = 1
self.boxFlag = 0
self.mMode = 2
self.innerRing = 200.0
self.outerRing = 250.0
self.innerLimit = 150.0
self.outerLimit = 300.0
self.angle = 0.0
self.mLeft = self.outerLimit
self.mRight = self.outerLimit
self.mTop = self.outerLimit
self.mBottom = self.outerLimit
self.circlePos[0,*] = COS(2.0*!PI*FINDGEN(32)/32)
self.circlePos[1,*] = SIN(2.0*!PI*FINDGEN(32)/32)
self.circleLines = INDGEN(33) - 1
self.circleLines[0] = 33
self.circleLines[33] = 0
RETURN, 1
END

PRO ldrsHALO::Update
self->ldrsIMGAREA::Update
; update the postion of the central cross
PRINT, 'ENTER ldrsHALO::Update'
size = 10.0
; update size and position of the ring circles
pos = self.circlePos*self.innerRing
pos[0, *] += self.pX
pos[1, *] += self.pY
IF NOT OBJ_VALID(self.innerRingObj) THEN BEGIN
    self.innerRingObj = OBJ_NEW('IDLgrPolyline', pos, $
                                POLYLINES=self.circleLines, $
                                THICK=2, LINESTYLE=2, COLOR=self.col)
    self->ADD, self.innerRingObj
ENDIF ELSE BEGIN
    self.innerRingObj->SetProperty, DATA=pos
ENDELSE
pos = self.circlePos*self.outerRing
pos[0, *] += self.pX
pos[1, *] += self.pY
IF NOT OBJ_VALID(self.outerRingObj) THEN BEGIN
    self.outerRingObj = OBJ_NEW('IDLgrPolyline', pos, $
                                POLYLINES=self.circleLines, $
                                THICK=2, LINESTYLE=2, COLOR=self.col)
    self->ADD, self.outerRingObj
ENDIF ELSE BEGIN
    self.outerRingObj->SetProperty, DATA=pos
ENDELSE
; update size and position of the limit circles
pos = self.circlePos*self.innerLimit
pos[0, *] += self.pX
pos[1, *] += self.pY
IF NOT OBJ_VALID(self.innerLimitObj) THEN BEGIN
    self.innerLimitObj = OBJ_NEW('IDLgrPolyline', pos, $
                                 POLYLINES=self.circleLines, $
                                 THICK=2, LINESTYLE=0, COLOR=self.col)
    self->ADD, self.innerLimitObj
ENDIF ELSE BEGIN
    self.innerLimitObj->SetProperty, DATA=pos
ENDELSE
pos = self.circlePos*self.outerLimit
pos[0, *] += self.pX
pos[1, *] += self.pY
IF NOT OBJ_VALID(self.outerLimitObj) THEN BEGIN
    self.outerLimitObj = OBJ_NEW('IDLgrPolyline', pos, $
                                 POLYLINES=self.circleLines, $
                                 THICK=2, LINESTYLE=0, COLOR=self.col)
    self->ADD, self.outerLimitObj
ENDIF ELSE BEGIN
    self.outerLimitObj->SetProperty, DATA=pos
ENDELSE
; update size and position of the direction marker
pos = [[self.pX, self.pY], $
       [self.pX + self.outerLimit*COS(!PI*self.angle/180.0), self.pY + self.outerLimit*SIN(!PI*self.angle/180.0)]]
IF NOT OBJ_VALID(self.angleObj) THEN BEGIN
    self.angleObj = OBJ_NEW('IDLgrPolyline', pos, $
                            THICK=2, LINESTYLE=0, COLOR=self.col)
    self->ADD, self.angleObj
ENDIF ELSE BEGIN
    self.angleObj->SetProperty, DATA=pos
ENDELSE
PRINT, 'LEAVE ldrsHALO::Update'
END

PRO ldrsHALO::AdjustCenter, proc
view = self.otool->GetView()
view->GetImage, img, nr
s = SIZE(*img)
nx = s[1]
ny = s[2]
sumx = 0.0
sumy = 0.0
sum  = 0.0
cx  = self.pX
cy  = self.pY
irr = self.innerRing
orr = self.outerRing
; loop in vertical direction
FOR j=-orr,orr DO BEGIN
    ; check if this line is inside the image
    y = cy + j
    IF y LT 0 THEN CONTINUE
    IF y GE ny THEN CONTINUE
    outerLeft = MAX([0.0, cx - ROUND(SQRT(orr*orr - j*j))])
    outerRight = MIN([nx - 1, cx + ROUND(SQRT(orr*orr - j*j))])
    IF (j LE irr) OR (j GE irr) THEN BEGIN
        ; no intersection with the inner ring
        FOR x = outerLeft,outerRight DO BEGIN
            val = (*img)[x, y, nr]
            sum += val
            sumx += x*val
            sumy += y*val
        ENDFOR
    ENDIF ELSE BEGIN
        ; intersection with the inner ring
        innerLeft = MAX([0.0, cx - ROUND(SQRT(irr*irr - j*j))])
        innerRight = MIN([nx - 1, cx + ROUND(SQRT(irr*irr - j*j))])
        ; left part
        IF innerLeft GE outerLeft THEN BEGIN
            FOR x = outerLeft,innerLeft DO BEGIN
                val = (*img)[x, y, nr]
                sum += val
                sumx += x*val
                sumy += y*val
            ENDFOR
        ENDIF
        ; right part
        IF innerRight LE outerRight THEN BEGIN
            FOR x = innerRight,outerRight DO BEGIN
                val = (*img)[x, y, nr]
                sum += val
                sumx += x*val
                sumy += y*val
            ENDFOR
        ENDIF
    ENDELSE
ENDFOR
self.pX = sumx/sum
self.pY = sumy/sum
PRINT, 'ldrsHALO::AdjustCenter, new center is at', self.pX, self.pY
self->Update
END

PRO ldrsHALO::GetRing, inner, outer
inner = self.innerRing
outer = self.outerRing
END

PRO ldrsHALO::SetRing, inner, outer
self.innerRing = inner
self.outerRing = outer
self->Update
END

PRO ldrsHALO::GetLimits, inner, outer
inner = self.innerLimit
outer = self.outerLimit
END

PRO ldrsHALO::SetLimits, inner, outer
self.innerLimit = inner
self.outerLimit = outer
self->Update
END

FUNCTION ldrsHALO::GetAngle
RETURN, self.angle
END

PRO ldrsHALO::SetAngle, angle
self.angle = angle
self->Update
END

PRO ldrsHALO::Enlarge
self->ldrsIMGAREA::Enlarge
self.innerRing /= 2
self.outerRing *= 2
self.innerLimit /= 2
self.outerLimit *= 2
self->Update
END

PRO ldrsHALO::Shrink
self->ldrsIMGAREA::Shrink
self.innerRing *= 2
self.outerRing /= 2
self.innerLimit *= 2
self.outerLimit /= 2
self->Update
END

PRO ldrsHALO::ChangeColor, col
self->ldrsIMGAREA::ChangeColor, col
self.innerRingObj->SetProperty, COLOR=col
self.outerRingObj->SetProperty, COLOR=col
self.innerLimitObj->SetProperty, COLOR=col
self.outerLimitObj->SetProperty, COLOR=col
self.angleObj->SetProperty, COLOR=col
END

PRO ldrsHALO::Store, cfg
self->ldrsIMGAREA::Store, cfg
cfg->SetFloat2D, 'RING',  self.innerRing, self.outerRing
cfg->SetFloat2D, 'LIMIT', self.innerLimit, self.outerLimit
cfg->SetFloat, 'ANGLE', self.angle
END

PRO ldrsHALO::Load, cfg
self->ldrsIMGAREA::Load, cfg
cfg->GetFloat2D, 'RING', x, y
self.innerRing = x
self.outerRing = y
cfg->GetFloat2D, 'LIMIT', x, y
self.innerLimit = x
self.outerLimit = y
self.angle = cfg->GetFloat('ANGLE', DEFAULT=0.0)
self->Update
END

PRO ldrsHALO__define
  struct = { ldrsHALO, $
             INHERITS ldrsIMGAREA, $
             innerLimit:0.0, outerLimit:0.0, $
             innerRing:0.0, outerRing:0.0, $
             angle:0.0, $
             circlePos:FLTARR(2,32), circleLines:INTARR(34), $
             innerRingObj:OBJ_NEW(), outerRingObj:OBJ_NEW(), $
             innerLimitObj:OBJ_NEW(), outerLimitObj:OBJ_NEW(), $
             angleObj:OBJ_NEW() $
           }
END
