;# Module
;MODULE<i>.TAG    = <string>          # unique module tag
;MODULE<i>.LABEL  = <string>          # label used in the GUI
;MODULE<i>.COLOR  = "<r> <g> <b>"     # color used in the flowchart
;MODULE<i>.BOX    = "<x> <y> <w> <h>" # rectangle in the flowchart
;MODULE<i>.IN<j>  = <string>          # tag of an input file
;MODULE<i>.OUT<j> = <string>          # tag on an output file
;# Algorithm
;MODULE<i>.ALGO<j>.TAG    = <string>  # unique algorithm tag
;MODULE<i>.ALGO<j>.LABEL  = <string>  # label used in the GUI
;MODULE<i>.ALGO<j>.CLASS  = <string>  # IDL class name (implementation)
;MODULE<i>.ALGO<j>.DIALOG = <string>  # IDL dialog function (implementation)
FUNCTION ldrsMODULE::Init, recipe, cfg
IF NOT self->ldrsNODE::Init(recipe, cfg) THEN RETURN, 0
; load all input filetypes (MODULE<i>.IN<j>) from the configuration file
hcfg = cfg->Locate('IN')
hcfg = hcfg->GetFirst()
WHILE OBJ_VALID(hcfg) DO BEGIN
    self.infiletypes[self.nInfiletypes] = recipe->FindFiletype(hcfg->GetString(''))
    self.nInfiletypes += 1
    hcfg = hcfg->GetNext()
ENDWHILE
; load all output filetypes (MODULE<i>.OUT<j>) from the configuration file
hcfg = cfg->Locate('OUT')
hcfg = hcfg->GetFirst()
WHILE OBJ_VALID(hcfg) DO BEGIN
    self.outfiletypes[self.nOutfiletypes] = recipe->FindFiletype(hcfg->GetString(''))
    self.nOutfiletypes += 1
    hcfg = hcfg->GetNext()
ENDWHILE
; load all algorithms (MODULE<i>.ALGO<j>) from the configuration file
child = cfg->Locate('ALGO')
child = child->GetFirst()
WHILE OBJ_VALID(child) DO BEGIN
    self.algorithms[self.nAlgorithms] = OBJ_NEW('ldrsALGORITHM', self, child)
    self.nAlgorithms += 1
    child = child->GetNext()
ENDWHILE
self.cfg = OBJ_NEW('ldrsCFG', OBJ_NEW(), '', -1)
self.used = 0
;PRINT, "ldrsMODULE::Init, ", self.tag
RETURN, 1
END

FUNCTION ldrsMODULE::GetNInFiletypes
RETURN, self.nInfiletypes
END

FUNCTION ldrsMODULE::GetInFiletype, nr
IF nr GE self.nInfiletypes THEN RETURN, OBJ_NEW()
RETURN, self.infiletypes[nr]
END

FUNCTION ldrsMODULE::GetNOutFiletypes
RETURN, self.nOutfiletypes
END

FUNCTION ldrsMODULE::GetOutFiletype, nr
IF nr GE self.nOutfiletypes THEN RETURN, OBJ_NEW()
RETURN, self.outfiletypes[nr]
END

FUNCTION ldrsMODULE::GetNAlgorithms
  RETURN, self.nAlgorithms
END

FUNCTION ldrsMODULE::GetAlgorithm, nr
IF nr GE self.nAlgorithms THEN RETURN, OBJ_NEW()
RETURN, self.algorithms[nr]
END

FUNCTION ldrsMODULE::FindAlgorithm, tag
FOR i = 0,self.nAlgorithms - 1 DO BEGIN
    IF self.algorithms[i]->GetTag() EQ tag THEN RETURN, self.algorithms[i]
END
RETURN, OBJ_NEW()
END

FUNCTION ldrsMODULE::GetSelectedAlgorithm
RETURN, self.selAlgorithm
END

PRO ldrsMODULE::SelectAlgorithm, algo
self.selAlgorithm = algo
END

FUNCTION ldrsMODULE::GetCFG
RETURN, self.cfg
END

FUNCTION ldrsMODULE::IsUsed
RETURN, self.used
END

PRO ldrsMODULE::SetUsed, flag
self.used = flag
END

PRO ldrsMODULE::Store, cfg
IF OBJ_VALID(self.selAlgorithm) THEN BEGIN
    self.cfg->SetString, 'ALGO', self.selAlgorithm->GetTag()
    ai = self.selAlgorithm->GetImpl()
    ai->Store, self.cfg
ENDIF
self.cfg->SetInt, 'USED', self.used
self.cfg->CloneChildren, cfg
END

PRO ldrsMODULE::Load, cfg
cfg->MoveChildren, self.cfg
algoTag = self.cfg->GetString('ALGO', DEFAULT='')
;PRINT, 'ldrsMODULE::Load, algoTag=', algoTag
IF algoTag NE '' THEN self.selAlgorithm = self->FindAlgorithm(algoTag)
IF OBJ_VALID(self.selAlgorithm) THEN BEGIN
    ai = self.selAlgorithm->GetImpl()
    ai->Load, self.cfg
ENDIF
self.used = self.cfg->GetInt('USED', DEFAULT=1)
END

PRO ldrsMODULE__define
struct = { ldrsMODULE, $
           INHERITS ldrsNODE, $
           nInfiletypes:0, $
           infiletypes:OBJARR(10), $
           nOutfiletypes:0, $
           outfiletypes:OBJARR(10), $
           nAlgorithms:0, $
           algorithms:OBJARR(10), $
           selAlgorithm:OBJ_NEW(), $
           cfg:OBJ_NEW(), $
           used:0 $
           }
END
