FUNCTION ldrsPOWERSPECTRUM::Init, algorithm
IF NOT self->ldrsALGOIMPL::Init(algorithm) THEN RETURN, 0
self.dlgTitle = 'Powerspectrum Setup'
self.preFT = self.recipe->FindFiletype('PRE')
self.objFT = self.recipe->FindFiletype('TAR')
self.psfFT = self.recipe->FindFiletype('PSF')
self.imgFT = self.recipe->FindFiletype('IMG')
RETURN, 1
END

FUNCTION ldrsPOWERSPECTRUM::Prepare
PRINT, '#################### ldrsPOWERSPECTRUM::Prepare'
IF NOT self->ldrsALGOIMPL::Prepare() THEN RETURN, 0
IF NOT self->CheckInputFiles(self.preFT, 0, 10) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.objFT, 0, 10) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.psfFT, 0, 10) THEN RETURN, 0
IF NOT self->CheckOutputFiles(self.imgFT, 1, 1) THEN RETURN, 0
self.nx = 0
self.ny = 0
self.nz = 0
lambda = 2.2
dlambda = 0.0
; determine the number of output images (one cube!)
nFiles = self.psfFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.psfFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
    cfg = file->GetFileCFG()
    hcfg = cfg->Locate('OBS.LAMBDA', /NO_CREATE)
    IF OBJ_VALID(hcfg) THEN lambda = hcfg->GetFloat('', DEFAULT=2.2)
    hcfg = cfg->Locate('OBS.DLAMBDA', /NO_CREATE)
    IF OBJ_VALID(hcfg) THEN dlambda = hcfg->GetFloat('', DEFAULT=0.0)
    IF file->GetNX() GT self.nx THEN self.nx = file->GetNX()
    IF file->GetNY() GT self.ny THEN self.ny = file->GetNY()
    self.nz += file->GetNZ()
ENDFOR
nFiles = self.objFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.objFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
    cfg = file->GetFileCFG()
    hcfg = cfg->Locate('OBS.LAMBDA', /NO_CREATE)
    IF OBJ_VALID(hcfg) THEN lambda = hcfg->GetFloat('', DEFAULT=2.2)
    hcfg = cfg->Locate('OBS.DLAMBDA', /NO_CREATE)
    IF OBJ_VALID(hcfg) THEN dlambda = hcfg->GetFloat('', DEFAULT=0.0)
    IF file->GetNX() GT self.nx THEN self.nx = file->GetNX()
    IF file->GetNY() GT self.ny THEN self.ny = file->GetNY()
    self.nz += file->GetNZ()
ENDFOR
; the preprocessed files are only used, when no extracted files do
; exist
IF self.nz EQ 0 THEN BEGIN
    nFiles = self.preFT->GetNFiles()
    FOR iFile=0,nFiles-1 DO BEGIN
        file = self.preFT->GetFile(iFile)
        IF file->LoadData() EQ 0 THEN RETURN, 0
        cfg = file->GetFileCFG()
        hcfg = cfg->Locate('OBS.LAMBDA', /NO_CREATE)
        IF OBJ_VALID(hcfg) THEN lambda = hcfg->GetFloat('', DEFAULT=2.2)
        hcfg = cfg->Locate('OBS.DLAMBDA', /NO_CREATE)
        IF OBJ_VALID(hcfg) THEN dlambda = hcfg->GetFloat('', DEFAULT=0.0)
        IF file->GetNX() GT self.nx THEN self.nx = file->GetNX()
        IF file->GetNY() GT self.ny THEN self.ny = file->GetNY()
        self.nz += file->GetNZ()
    ENDFOR
ENDIF
PRINT, ' input = [', self.nx, ',', self.ny, ',', self.nz, '] pixels at ', lambda, ' um, width ', dlambda, ' um'
; create output cube
self.imgFile = self.imgFT->GetFile(0)
self.imgFile->CreateData, self.nx, self.ny, self.nz, /FORCE
cfg = self.imgFile->GetFileCFG()
cfg->SetFloat, 'OBS.LAMBDA', lambda
cfg->SetFloat, 'OBS.DLAMBDA', dlambda
self.img = self.imgFile->GetData()
self->Load
RETURN, 1
END

PRO ldrsPOWERSPECTRUM::ProcessImage, z, nx, ny, img, i
xo = (self.nx - nx)/2
yo = (self.ny - ny)/2
ps = FFT((*img)[*,*,i])

(*self.img)[xo : xo + nx - 1, yo : yo + ny - 1, z] = SHIFT(ABS(ps)^2, nx/2, ny/2)
z += 1
END

FUNCTION ldrsPOWERSPECTRUM::Execute
z = 0
; calculate the powerspectra for the extracted calibrator images
nFiles = self.psfFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.psfFT->GetFile(iFile)
    nx = file->GetNX()
    ny = file->GetNY()
    nz = file->GetNZ()
    data = file->GetData()
;    PRINT, '  file ', file->GetName(), ' => [', nx, ',', ny, ',', nz, '], offset=(', xo, ',', yo, ')'
    FOR i=0,nz-1 DO BEGIN
        self->ProcessImage, z, nx, ny, data, i
    ENDFOR
ENDFOR
; calculate the powerspectra for the extracted target images
nFiles = self.objFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.objFT->GetFile(iFile)
    nx = file->GetNX()
    ny = file->GetNY()
    nz = file->GetNZ()
    data = file->GetData()
;    PRINT, '  file ', file->GetName(), ' => [', nx, ',', ny, ',', nz, '], offset=(', xo, ',', yo, ')'
    FOR i=0,nz-1 DO BEGIN
        self->ProcessImage, z, nx, ny, data, i
    ENDFOR
ENDFOR
; the preprocessed files are only used, when no extracted files do
; exist
IF z EQ 0 THEN BEGIN
    ; calculate the powerspectra for the preprocessed images
    nFiles = self.preFT->GetNFiles()
    FOR iFile=0,nFiles-1 DO BEGIN
        file = self.preFT->GetFile(iFile)
        nx = file->GetNX()
        ny = file->GetNY()
        nz = file->GetNZ()
        data = file->GetData()
;        PRINT, '  file ', file->GetName(), ' => [', nx, ',', ny, ',', nz, '], offset=(', xo, ',', yo, ')'
        FOR i=0,nz-1 DO BEGIN
            self->ProcessImage, z, nx, ny, data, i
        ENDFOR
    ENDFOR
ENDIF
(*self.img)[self.nx/2, self.ny/2, *] = 0.0
RETURN, 1
END


FUNCTION ldrsPOWERSPECTRUM::Finish
;PRINT, 'ldrsPOWERSPECTRUM::Finish'
IF NOT self->ldrsALGOIMPL::Finish() THEN RETURN, 0
result = self.imgFile->StoreData()
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.img, NAME='Powerspectrum'
RETURN, 1
END

PRO ldrsPOWERSPECTRUM__define
struct = { ldrsPOWERSPECTRUM, $
           INHERITS ldrsALGOIMPL, $
           preFT:OBJ_NEW(), $
           objFT:OBJ_NEW(), $
           psfFT:OBJ_NEW(), $
           imgFT:OBJ_NEW(), $
           imgFile:OBJ_NEW(), $
           img:PTR_NEW(), $
           nx:0, ny:0, nz:0 $
         }
END
