FUNCTION ldrsPSFARTIFICIAL::Init, algorithm
IF NOT self->ldrsEXTRACTION::Init(algorithm) THEN RETURN, 0
self.content = 2 ; we need the intermediate target as a reference!
self.dlgTitle = 'Artificial PSF Setup'
self.intFT = self.recipe->FindFiletype('PRE')
self.psfFT = self.recipe->FindFiletype('PSF')
self.nResults = 2
FOR i=0,N_ELEMENTS(self.dCoeff)-1 DO self.coeff[i] = 2.0
RETURN, 1
END

FUNCTION ldrsPSFARTIFICIAL::Prepare
PRINT, 'ldrsPSFARTIFICIAL::Prepare'
IF NOT self->ldrsEXTRACTION::Prepare() THEN RETURN, 0
IF NOT self->CheckInputFiles(self.intFT, 1, 10) THEN RETURN, 0
; we need exactly one psf file which contains all psfs
IF NOT self->CheckOutputFiles(self.psfFT, 1, 1) THEN RETURN, 0
RETURN, 1
END

PRO ldrsPSFARTIFICIAL::ProcImage, srcData, iImage, partData, nx, ny, nz, z, pa, lambda, dlambda
beta = 1.2
aosize = self.seeing/0.005089/2 ; 5mas per pixel, radius of the seeing disc
rho2 = aosize*aosize/(2.0^(1.0/beta) - 1.0)
PRINT, '  ProcImage:', iImage, nx, ny, nz, z, pa, lambda, aosize, SQRT(rho2)
;PRINT, 'ProcImage: total before=', total((*partData)[*, *, z])
psfpart = FLTARR(nx, ny)
LDRS_CREATE_PSF, nx, ny, pa/!RADEG, lambda, dlambda, psfpart
psfpart *= self.strehl/100.0/TOTAL(psfpart)
(*partData)[*, *, z] += psfpart
IF self.strehl NE 100.0 THEN BEGIN
    aopart = FLTARR(nx, ny)
    FOR x=0, nx-1 DO BEGIN
        FOR y=0, ny-1 DO BEGIN
            r = SQRT((nx/2 - x)^2 + (ny/2 - y)^2)/aosize
            ;aopart[x,y] = exp(-0.5*r^1.5)
            ;r2 = (nx/2 - x)^2 + (ny/2 - y)^2
            aopart[x,y] = self.coeff[0]*exp(-(r/self.coeff[1])^self.coeff[2]) + self.coeff[3]*exp(-(r/self.coeff[4])^self.coeff[5])
            ;aopart[x,y] = (r2/rho2 + 1.0)^(-beta)
        ENDFOR
    ENDFOR
    aopart *= (1.0 - self.strehl/100.0)/TOTAL(aopart)
    (*partData)[*, *, z] += aopart
ENDIF
IF self.nResults EQ 3 THEN z++
;PRINT, 'ProcImage: total after=', total((*partData)[*, *, z])
END

PRO ldrsPSFARTIFICIAL::ProcFile, srcFile, dstFile, partData, nx, ny, nz, z
PRINT, '  ProcFile: ', srcFile->GetName(), ', ', dstFile->GetName(), ', ,', nx, ny, nz, z
; iterate over all images in this file
srcData = srcFile->GetData()
nImages = srcFile->GetNZ()
cfg = srcFile->GetFileCFG()
hcfg = cfg->Locate('OBS.LAMBDA', /NO_CREATE)
IF OBJ_VALID(hcfg) THEN BEGIN
    lambda = hcfg->GetFloat('', DEFAULT=2.2)
    hcfg = dstFile->GetFileCFG()
    hcfg->SetFloat, 'OBS.LAMBDA', lambda
ENDIF ELSE BEGIN
    lambda = 2.2
ENDELSE
hcfg = cfg->Locate('OBS.DLAMBDA', /NO_CREATE)
IF OBJ_VALID(hcfg) THEN BEGIN
    dlambda = hcfg->GetFloat('', DEFAULT=0.0)
    hcfg = dstFile->GetFileCFG()
    hcfg->SetFloat, 'OBS.DLAMBDA', dlambda
ENDIF ELSE BEGIN
    dlambda = 0.0
ENDELSE
pa = 0.0
; loop over all interferograms in order to build the part image
FOR iImage=0,nImages-1 DO BEGIN
    ; check if this image has a part star specification
    ; if yes => use the new one
    cfg = srcFile->GetImageCFG(iImage)
    IF OBJ_VALID(cfg) THEN BEGIN
        rotator = cfg->GetFloat('OBS.ROTATOR', DEFAULT=0.0)
        rotate  = cfg->GetFloat('PREP.ROTATE', DEFAULT=0.0)
        pa = rotator + rotate
    ENDIF
    self->ProcImage, srcData, iImage, partData, nx, ny, nz, z, pa, lambda, dlambda
    IF self.nResults EQ 2 THEN BEGIN
        hhcfg = dstFile->GetImageCFG(z)
        hhcfg->SetFloat, 'BGRD', self.bgrd
        self.bgrd = 0.0
        z++
    ENDIF
ENDFOR
; show the ceated part
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=partData, NAME='PART'
WAIT, 2
RETURN
END

FUNCTION ldrsPSFARTIFICIAL::Execute
PRINT, 'ldrsPSFARTIFICIAL::Execute:'
; calculate the dimensions of the result
srcFT = self.intFT
dstFT = self.psfFT
self->CalcDimensions, srcFT, 'TARGET', nx, ny, nz
IF (nx EQ 0 ) OR (ny EQ 0) OR (nz EQ 0) THEN RETURN, 0
; create the part image (nx, ny, nz)
dstFile = dstFT->GetFile(0)
dstFile->CreateData, nx, ny, nz, /FORCE
partData = dstFile->GetData()
IF NOT PTR_VALID(partData) THEN RETURN, 0
(*partData)[*] = 0.0
; loop over all files
nFiles = srcFT->GetNFiles()
z = 0
FOR iFile=0,nFiles-1 DO BEGIN
    srcFile = srcFT->GetFile(iFile)
    IF NOT srcFile->CheckContent(self.content) THEN CONTINUE
    self->ProcFile, srcFile, dstFile, partData, nx, ny, nz, z
    IF self.nResults EQ 1 THEN  BEGIN
        hhcfg = dstFile->GetImageCFG(z)
        hhcfg->SetFloat, 'BGRD', self.bgrd
        self.bgrd = 0.0
        z++
    ENDIF
ENDFOR
IF self.nResults EQ 0 THEN  BEGIN
    hhcfg = dstFile->GetImageCFG(z)
    hhcfg->SetFloat, 'BGRD', self.bgrd
    self.bgrd = 0.0
ENDIF
RETURN, 1
END

FUNCTION ldrsPSFARTIFICIAL::Finish
PRINT, 'ldrsPSFARTIFICIAL::Finish'
IF NOT self->ldrsEXTRACTION::Finish() THEN RETURN, 0
RETURN, self->StoreOutput(self.psfFT)
END

PRO ldrsPSFARTIFICIAL::Load, cfg
self->ldrsEXTRACTION::Load, cfg
IF OBJ_VALID(cfg) THEN BEGIN
    self.nResults = cfg->GetInt('NRESULTS', DEFAULT=2)
    self.seeing = cfg->GetFloat('SEEING', DEFAULT=0.3)
    self.strehl = cfg->GetFloat('STREHL', DEFAULT=20.0)
    hcfg = cfg->LocateByName('COEFF', /NO_CREATE)
    IF OBJ_VALID(hcfg) THEN BEGIN
        FOR i=0,N_ELEMENTS(self.dCoeff)-1 DO BEGIN
            hhcfg = hcfg->LocateByNr(i+1, /NO_CREATE)
            IF OBJ_VALID(hhcfg) THEN self.coeff[i] = hhcfg->GetFloat('', DEFAULT=2.0)
        ENDFOR
    ENDIF
ENDIF ELSE BEGIN
    self.nResults = 2
    self.seeing = 0.3
    self.strehl = 20.0
    FOR i=0,N_ELEMENTS(self.dCoeff)-1 DO self.coeff[i] = 2.0
ENDELSE
END

PRO ldrsPSFARTIFICIAL::Store, cfg
self->ldrsEXTRACTION::Store, cfg
IF OBJ_VALID(cfg) THEN BEGIN
    cfg->SetInt, 'NRESULTS', self.nResults
    cfg->SetFloat, 'SEEING', self.seeing
    cfg->SetFloat, 'STREHL', self.strehl
    hcfg = cfg->LocateByName('COEFF')
    FOR i=0,N_ELEMENTS(self.dCoeff)-1 DO BEGIN
        hhcfg = hcfg->LocateByNr(i+1)
        hhcfg->SetFloat, '', self.coeff[i]
    ENDFOR
ENDIF
END

PRO ldrsPSFARTIFICIAL::ItemSelected, id, item
IF id EQ self.dNResults THEN self.nResults = item
END

PRO ldrsPSFARTIFICIAL::ReadDynamicContent
self->ldrsEXTRACTION::ReadDynamicContent
WIDGET_CONTROL, self.dNResults, SET_COMBOBOX_SELECT=self.nResults
WIDGET_CONTROL, self.dSeeing, SET_VALUE=STRING(self.seeing)
WIDGET_CONTROL, self.dStrehl, SET_VALUE=STRING(self.strehl)
FOR i=0,N_ELEMENTS(self.dCoeff)-1 DO $
  WIDGET_CONTROL, self.dCoeff[i], SET_VALUE=STRING(self.coeff[i])
END

PRO ldrsPSFARTIFICIAL::WriteDynamicContent
self->ldrsEXTRACTION::WriteDynamicContent
WIDGET_CONTROL, self.dSeeing, GET_VALUE=str
self.seeing = FLOAT(str)
WIDGET_CONTROL, self.dStrehl, GET_VALUE=str
self.strehl = FLOAT(str)
FOR i=0,N_ELEMENTS(self.dCoeff)-1 DO BEGIN
    WIDGET_CONTROL, self.dCoeff[i], GET_VALUE=str
    self.coeff[i] = FLOAT(str)
ENDFOR
END

PRO ldrsPSFARTIFICIAL::CreateContent
self->ldrsEXTRACTION::CreateContent
row = self->GetFreeRow()
lab              = self->CreateLabel(0, row, 'Add:', /HALIGN_RIGHT)
self.dNResults   = self->CreateComboBox(1, row, ['One', 'File', 'Image', 'Object'])
row++
lab              = self->CreateLabel(0, row, 'Seeing [arcsec]:', /HALIGN_RIGHT)
self.dSeeing     = self->CreateEntry(1, row, VALUE='0.3')
row++
lab              = self->CreateLabel(0, row, 'Strehl [%]:', /HALIGN_RIGHT)
self.dStrehl     = self->CreateEntry(1, row, VALUE='20.0')
row++
lab              = self->CreateLabel(0, row, 'Coeff:', /HALIGN_RIGHT)
FOR i=0,N_ELEMENTS(self.dCoeff)-1 DO $
  self.dCoeff[i]   = self->CreateEntry((i MOD 3) + 1, row + i/3, VALUE='2.0')
row += (N_ELEMENTS(self.dCoeff) + 2)/3
END

PRO ldrsPSFARTIFICIAL__define
struct = { ldrsPSFARTIFICIAL, $
           INHERITS ldrsEXTRACTION, $
           dNResults:0, dSeeing:0, dStrehl:0, dCoeff:INTARR(6), $
           seeing:0.0, strehl:0.0, coeff:DBLARR(6), $
           intFT:OBJ_NEW(), $
           psfFT:OBJ_NEW() $
         }
END
