FUNCTION ldrsPSFCREATION::Init, algorithm
IF NOT self->ldrsEXTRACTION::Init(algorithm) THEN RETURN, 0
self.content = 1
self.dlgTitle = 'PSF Creation Setup'
self.intFT = self.recipe->FindFiletype('PRE')
self.psfFT = self.recipe->FindFiletype('PSF')
self.nResults = 2
RETURN, 1
END

FUNCTION ldrsPSFCREATION::Prepare
PRINT, 'ldrsPSFCREATION::Prepare'
IF NOT self->ldrsEXTRACTION::Prepare() THEN RETURN, 0
IF NOT self->CheckInputFiles(self.intFT, 1, 10) THEN RETURN, 0
; we need exactly one psf file which contains all psfs
IF NOT self->CheckOutputFiles(self.psfFT, 1, 1) THEN RETURN, 0
RETURN, 1
END

FUNCTION ldrsPSFCREATION::Execute
PRINT, 'ldrsPSFCREATION::Execute:'
; use the dimensions of the target as the PSF size
; this means that sometimes the missing pixels (if the PSFSTAR is
; smaller) have to be extrapolated (procedure PSFEXT)
self->CalcDimensions, self.intFT, 'TARGET', nx, ny, nz
self.dstAutoDim = 0
self.dstNX = nx
self.dstNY = ny
self.dstCX = nx/2
self.dstCY = ny/2
RETURN, self->ExtractPart(self.intFT, 'PSFSTAR', self.psfFT)
END

FUNCTION ldrsPSFCREATION::Finish
PRINT, 'ldrsPSFCREATION::Finish'
IF NOT self->ldrsEXTRACTION::Finish() THEN RETURN, 0
RETURN, self->StoreOutput(self.psfFT)
END

PRO ldrsPSFCREATION::Load, cfg
self->ldrsEXTRACTION::Load, cfg
IF OBJ_VALID(cfg) THEN BEGIN
    self.nResults = cfg->GetInt('NRESULTS', DEFAULT=2)
    self.removeBgrd = cfg->GetInt('REMOVEBGRD', DEFAULT=0)
    self.useMask = cfg->GetInt('USEFFMASK', DEFAULT=0)
    self.recenter = cfg->GetInt('RECENTER', DEFAULT=0)
ENDIF ELSE BEGIN
    self.nResults = 2
    self.removeBgrd = 0
    self.useMask = 0
    self.recenter = 0
ENDELSE
END

PRO ldrsPSFCREATION::Store, cfg
self->ldrsEXTRACTION::Store, cfg
IF OBJ_VALID(cfg) THEN BEGIN
    cfg->SetInt, 'NRESULTS', self.nResults
    cfg->SetInt, 'REMOVEBGRD', self.removeBgrd
    cfg->SetInt, 'USEFFMASK', self.useMask
    cfg->SetInt, 'RECENTER', self.recenter
ENDIF
END

PRO ldrsPSFCREATION::ItemSelected, id, item
IF id EQ self.dNResults THEN self.nResults = item
END

PRO ldrsPSFCREATION::ReadDynamicContent
self->ldrsEXTRACTION::ReadDynamicContent
WIDGET_CONTROL, self.dNResults, SET_COMBOBOX_SELECT=self.nResults
WIDGET_CONTROL, self.dRemoveBgrd, SET_VALUE=[self.removeBgrd]
WIDGET_CONTROL, self.dUseMask, SET_VALUE=[self.useMask]
WIDGET_CONTROL, self.dRecenter, SET_VALUE=[self.recenter]
END

PRO ldrsPSFCREATION::WriteDynamicContent
self->ldrsEXTRACTION::WriteDynamicContent
WIDGET_CONTROL, self.dRemoveBgrd, GET_VALUE=flags
self.removeBgrd = flags[0]
WIDGET_CONTROL, self.dUseMask, GET_VALUE=flags
self.useMask = flags[0]
WIDGET_CONTROL, self.dRecenter, GET_VALUE=flags
self.recenter = flags[0]
END

PRO ldrsPSFCREATION::CreateContent
self->ldrsEXTRACTION::CreateContent
row = self->GetFreeRow()
lab              = self->CreateLabel(0, row, 'Add:', /HALIGN_RIGHT)
self.dNResults   = self->CreateComboBox(1, row, ['One', 'File', 'Image', 'Object'])
row++
lab              = self->CreateLabel(0, row, 'Background:', /HALIGN_RIGHT)
self.dRemoveBgrd = self->CreateCheckbox(1, row, 'Remove')
row++
lab              = self->CreateLabel(0, row, 'Fourier Mask:', /HALIGN_RIGHT)
self.dUseMask    = self->CreateCheckbox(1, row, 'Use')
row++
lab              = self->CreateLabel(0, row, 'Centering:', /HALIGN_RIGHT)
self.dRecenter   = self->CreateCheckbox(1, row, 'Recenter')
row++
END

PRO ldrsPSFCREATION__define
struct = { ldrsPSFCREATION, $
           INHERITS ldrsEXTRACTION, $
           dNResults:0, dRemoveBgrd:0, dUseMask:0, dRecenter:0, $
           intFT:OBJ_NEW(), $
           psfFT:OBJ_NEW() $
         }
END
