FUNCTION ldrsPSFFITTING::Init, algorithm
IF NOT self->ldrsPSFCREATION::Init(algorithm) THEN RETURN, 0
;self.content = 1
self.dlgTitle = 'PSF Fitting Setup'
;self.intFT = self.recipe->FindFiletype('PRE')
;self.psfFT = self.recipe->FindFiletype('PSF')
;self.nResults = 2
RETURN, 1
END

;FUNCTION ldrsPSFFITTING::Prepare
;PRINT, 'ldrsPSFFITTING::Prepare'
;IF NOT self->ldrsPSFCREATION::Prepare() THEN RETURN, 0
;IF NOT self->CheckInputFiles(self.intFT, 1, 10) THEN RETURN, 0
;; we need exactly one psf file which contains all psfs
;IF NOT self->CheckOutputFiles(self.psfFT, 1, 1) THEN RETURN, 0
;RETURN, 1
;END

; The PSF halo is fittet against a gaussian function with the
; following form:
; halo(r) := a*EXP(-(r/b)^c)
; the partial derivatives (for fitting with LMFIT) are:
; d/da := EXP(-(r/b)^c)
; d/db := a*EXP(-(r/b)^c)*(c*(x/b)^c/b)
; d/dc := a*EXP(-(r/b)^c)*(-(r/b)^c)*ln(x/b)
; For the first tests, the value for c will be fixed to 1.6
FUNCTION ldrsHaloFit, X, A
bx = (X/A[1])^A[2]
RETURN, [ [A[0]*EXP(-bx)], $
          [EXP(-bx)], $
          [A[0]*A[2]*EXP(-bx)*bx/A[1]], $
          [A[0]*EXP(-bx)*(-bx)*ALOG(X/A[1])] $
        ]
END
FUNCTION ldrsHalo, X, A
RETURN, A[0]*EXP(-(X/A[1])^A[2])
END

FUNCTION ldrsPSFFITTING::Execute
PRINT, 'ldrsPSFFITTING::Execute:'
IF NOT self->ExtractPart(self.intFT, 'PSFSTAR', self.psfFT) THEN RETURN, 0
; use the extracted PSFs for fitting and creating an artificial one
; ..
PRINT, 'fit and create ', self.dstCount, ' PSFs'
i = 0
FOR j=0,self.dstCount-1 DO BEGIN
    PRINT, '  destination ', j, ' contains ', self.dstSrcCounts[j], ' sources'
    FOR k=0,self.dstSrcCounts[j]-1 DO BEGIN
        PRINT, '    source ', i, ' pa=', self.srcPA[i], ', lambda=', self.srcLambda[i], ', dlambda=', self.srcDLambda[i]
        i++
    ENDFOR
ENDFOR
dstFile = self.psfFT->GetFile(0)
partData = dstFile->GetData() ; (*partData)[*, *, z]
s = SIZE(*partData, /DIMENSIONS)
nx = s[0]
ny = s[1]
obs_lambda = self.srcLambda[0]
det_pix_size=18.0
; rf := Skalierungsfaktor fuer den Spiegeldurchmesser
; rh := Skalierungsfaktor fuer das zentrale Loch im Spiegel
; af := Periode fuer den Cosinus als Ueberlagerung
; fh := Flaechenverhaeltnis Spiegel zu zentralem Loch
; r  := X-Koordinate (wg. der Ueberlagerung)
; psf(r) = (J1(PI*r/rf)/(PI*r/rf) - fh*J1(PI*r/rh)/(PI*r/rh))^2 * (1 + cos(PI*r/af))
; Die Nullstellen dieser Funktion dienen dazu, das Halo an diesen
; Positionen zu bestimmen
rf  = obs_lambda/det_pix_size*32.57*22.8/8.4 ; scale factor for J1(r)
rh  = obs_lambda/det_pix_size*32.57*22.8/1.0  ; scale factor for J1(r)
af  = 1.236*obs_lambda/det_pix_size*32.57*14.4/22.8 ; scale factor for cos(a)
fh  = (0.5/4.2)^2
PRINT, '  rf=', rf
PRINT, '  af=', af
; zero points of J1 (from Mathematica)
zpl = [3.8317059702075120242, 7.0155866698156268413, $
       10.173468135062721629, 13.323691936314222772, $
       16.470630050877634432, 19.615858510468243026, $
       22.760084380592772391, 25.903672087618382136, $
       29.046828534916855347, 32.189679910974405175, $
       35.332307550083868364, 38.474766234771614393, $
       41.617094212814450316, 44.759318997652819405, $
       47.901460887185450588, 51.043535183571506764, $
       54.185553641061318331, 57.327525437901009298, $
       60.469457845347491798, 63.611356698481230865 $
      ]
; calculate the number of zero points
zpcount = 0
FOR i=0,N_ELEMENTS(zpl)-1 DO BEGIN
    r = CEIL(zpl[i]*rf/!PI)
    IF r LT nx/2 THEN zpcount++
END
PRINT, '  using ', zpcount, ' zero points of J1'
;router = CEIL(7*rf/!PI)
adradius = CEIL(zpl[6]*rf/!PI)
halostart = CEIL(zpl[3]*rf/!PI)
PRINT, '  outer airy-disc diameter ', adradius, ' halo start ', halostart
;apx = INDGEN(nx/2-halostart, /FLOAT) + halostart
;apy = FLTARR(nx/2-halostart)
apx = FLTARR(zpcount)
apy = FLTARR(zpcount)
sz = 0
FOR z=0,self.dstCount-1 DO BEGIN
    PRINT, '  destination ', z
    ; calculate the radial plot (average)
    ;apy[*] = 0
;    PRINT, '    radial plot (extracted):'
;    FOR r=0,nx/2-1 DO BEGIN
;        ;apy[r-halostart] = 0.0
;        sum = 0.0D
;        n = FIX(2.0*!PI*r + 1)
;        FOR l=0,n-1 DO BEGIN
;            alpha = 2.0*!PI*FLOAT(l)/FLOAT(n)
;            ;apy[r-halostart] += (*partData)[nx/2 + ROUND(r*cos(alpha)), ny/2 + ROUND(r*sin(alpha)), z]
;            sum += (*partData)[nx/2 + ROUND(r*cos(alpha)), ny/2 + ROUND(r*sin(alpha)), z]
;        ENDFOR
;        ;apy[r-halostart] = apy[r-halostart]/n
;        PRINT, r, sum/n
;    ENDFOR
;    PRINT, '    profile plot (extracted):'
;    FOR r=0,nx/2-1 DO BEGIN
;        PRINT, r, (*partData)[nx/2 + r, ny/2, z]
;    ENDFOR
;    PRINT, '    halo data:'
    FOR i=0,zpcount-1 DO BEGIN
        r = zpl[i]*rf/!PI
        apx[i] = r
        n = FIX(2.0*!PI*r + 1)
        apy[i] = 0.0
        FOR l=0,n-1 DO BEGIN
            alpha = 2.0*!PI*FLOAT(l)/FLOAT(n)
            apy[i] += (*partData)[nx/2 + ROUND(r*cos(alpha)), ny/2 + ROUND(r*sin(alpha)), z]
        ENDFOR
        apy[i] = apy[i]/n
;        PRINT, apx[i], apy[i]
    END
    ; fit a gaussian halo to the data
    A = [apy[0], 30.0, 1.6]
    AR = LMFIT(apx, apy, A, FITA=[1, 1, 1], FUNCTION_NAME='ldrsHaloFit', /DOUBLE)
    PRINT, '    halo fit ', A
    ; create the halo part of the PSF
    halopart = FLTARR(nx, ny)
    FOR x=0,nx-1 DO BEGIN
        FOR y=0,ny-1 DO BEGIN
            r=SQRT((x-nx/2)*(x-nx/2) + (y-ny/2)*(y-ny/2))
            halopart[x,y] = ldrsHalo(r, A)
        ENDFOR
    ENDFOR
    halototal = TOTAL(halopart, /DOUBLE)
    ; create the ideal part of the PSF
    corepart = FLTARR(nx, ny)
    hcorepart = FLTARR(nx, ny)
    FOR k=0,self.dstSrcCounts[z]-1 DO BEGIN
        LDRS_CREATE_PSF, nx, ny, self.srcPA[sz]/!RADEG, self.srcLambda[sz], self.srcDLambda[sz], hcorepart
        corepart += hcorepart
        sz++
    ENDFOR
    ; calculate the scaling factor for the ideal PSF
    alpha = TOTAL(((*partData)[*, *, z] - halopart[*, *])*corepart[*, *], /DOUBLE) / $
      TOTAL(corepart[*, *]^2, /DOUBLE)
;    alpha = TOTAL(((*partData)[*, *, z] - halopart[*, *])*(corepart[*, *]/(*partData)[*, *, z]), /DOUBLE) / $
;      TOTAL((corepart[*, *]^2)/(*partData)[*, *, z], /DOUBLE)
    PRINT, '   core fit, alpha=', alpha
    PRINT, '   => strehl=', 100.0*alpha*TOTAL(corepart[*, *], /DOUBLE)/TOTAL((*partData)[*, *, z], /DOUBLE)
;    (*partData)[*, *, z] = halopart
;    (*partData)[*, *, z] = corepart
    (*partData)[*, *, z] = corepart*alpha + halopart
;    PRINT, '    radial plot (modelled):'
;    FOR r=0,nx/2-1 DO BEGIN
;        ;apy[r-halostart] = 0.0
;        sum = 0.0D
;        n = FIX(2.0*!PI*r + 1)
;        FOR l=0,n-1 DO BEGIN
;            alpha = 2.0*!PI*FLOAT(l)/FLOAT(n)
;            ;apy[r-halostart] += (*partData)[nx/2 + ROUND(r*cos(alpha)), ny/2 + ROUND(r*sin(alpha)), z]
;            sum += (*partData)[nx/2 + ROUND(r*cos(alpha)), ny/2 + ROUND(r*sin(alpha)), z]
;        ENDFOR
;        ;apy[r-halostart] = apy[r-halostart]/n
;        PRINT, r, sum/n
;    ENDFOR
;    PRINT, '    profile plot (modelled):'
;    FOR r=0,nx/2-1 DO BEGIN
;        PRINT, r, (*partData)[nx/2 + r, ny/2, z]
;    ENDFOR
    CONTINUE
ENDFOR
; show the ceated part
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=partData, NAME='Modelled PSF'
WAIT, 2
RETURN, 1
END

;FUNCTION ldrsPSFFITTING::Finish
;PRINT, 'ldrsPSFFITTING::Finish'
;IF NOT self->ldrsPSFCREATION::Finish() THEN RETURN, 0
;RETURN, self->StoreOutput(self.psfFT)
;END

;PRO ldrsPSFFITTING::Load, cfg
;self->ldrsPSFCREATION::Load, cfg
;IF OBJ_VALID(cfg) THEN BEGIN
;    self.nResults = cfg->GetInt('NRESULTS', DEFAULT=2)
;    self.removeBgrd = cfg->GetInt('REMOVEBGRD', DEFAULT=0)
;    self.useMask = cfg->GetInt('USEFFMASK', DEFAULT=0)
;    self.recenter = cfg->GetInt('RECENTER', DEFAULT=0)
;ENDIF ELSE BEGIN
;    self.nResults = 2
;    self.removeBgrd = 0
;    self.useMask = 0
;    self.recenter = 0
;ENDELSE
;END

;PRO ldrsPSFFITTING::Store, cfg
;self->ldrsPSFCREATION::Store, cfg
;IF OBJ_VALID(cfg) THEN BEGIN
;    cfg->SetInt, 'NRESULTS', self.nResults
;    cfg->SetInt, 'REMOVEBGRD', self.removeBgrd
;    cfg->SetInt, 'USEFFMASK', self.useMask
;    cfg->SetInt, 'RECENTER', self.recenter
;ENDIF
;END

;PRO ldrsPSFFITTING::ItemSelected, id, item
;IF id EQ self.dNResults THEN self.nResults = item
;END

;PRO ldrsPSFFITTING::ReadDynamicContent
;self->ldrsPSFCREATION::ReadDynamicContent
;WIDGET_CONTROL, self.dNResults, SET_COMBOBOX_SELECT=self.nResults
;WIDGET_CONTROL, self.dRemoveBgrd, SET_VALUE=[self.removeBgrd]
;WIDGET_CONTROL, self.dUseMask, SET_VALUE=[self.useMask]
;WIDGET_CONTROL, self.dRecenter, SET_VALUE=[self.recenter]
;END

;PRO ldrsPSFFITTING::WriteDynamicContent
;self->ldrsPSFCREATION::WriteDynamicContent
;WIDGET_CONTROL, self.dRemoveBgrd, GET_VALUE=flags
;self.removeBgrd = flags[0]
;WIDGET_CONTROL, self.dUseMask, GET_VALUE=flags
;self.useMask = flags[0]
;WIDGET_CONTROL, self.dRecenter, GET_VALUE=flags
;self.recenter = flags[0]
;END

;PRO ldrsPSFFITTING::CreateContent
;self->ldrsPSFCREATION::CreateContent
;row = self->GetFreeRow()
;lab              = self->CreateLabel(0, row, 'Add:', /HALIGN_RIGHT)
;self.dNResults   = self->CreateComboBox(1, row, ['One', 'File', 'Image', 'Object'])
;row++
;lab              = self->CreateLabel(0, row, 'Background:', /HALIGN_RIGHT)
;self.dRemoveBgrd = self->CreateCheckbox(1, row, 'Remove')
;row++
;lab              = self->CreateLabel(0, row, 'Fourier Mask:', /HALIGN_RIGHT)
;self.dUseMask    = self->CreateCheckbox(1, row, 'Use')
;row++
;lab              = self->CreateLabel(0, row, 'Centering:', /HALIGN_RIGHT)
;self.dRecenter   = self->CreateCheckbox(1, row, 'Recenter')
;row++
;END

PRO ldrsPSFFITTING__define
struct = { ldrsPSFFITTING, $
           INHERITS ldrsPSFCREATION, $
           dummy:0 $
         }
END
