;
; Syntax of the configuration file:
;##########################
;# Module
;MODULE<i>.TAG    = <string>          # unique module tag
;MODULE<i>.LABEL  = <string>          # label used in the GUI
;MODULE<i>.COLOR  = "<r> <g> <b>"     # color used in the flowchart
;MODULE<i>.BOX    = "<x> <y> <w> <h>" # rectangle in the flowchart
;MODULE<i>.IN<j>  = <string>          # tag of an input file
;MODULE<i>.OUT<j> = <string>          # tag on an output file
;# Algorithm
;MODULE<i>.ALGO<j>.TAG    = <string>  # unique algorithm tag
;MODULE<i>.ALGO<j>.LABEL  = <string>  # label used in the GUI
;MODULE<i>.ALGO<j>.CLASS  = <string>  # IDL class name (implementation)
;MODULE<i>.ALGO<j>.DIALOG = <string>  # IDL dialog function (implementation)
;##########################
;# File
;FILE<i>.TAG     = <string>           # unique file tag
;FILE<i>.LABEL   = <string>           # label used in the GUI
;FILE<i>.COLOR   = "<r> <g> <b>"      # color used in the flowchart
;FILE<i>.BOX     = "<x> <y> <w> <h>"  # rectangle in the flowchart
;FILE<i>.MINCARD = <int>              # minimum number of files
;FILE<i>.MAXCARD = <int>              # maximum number of files

FUNCTION ldrsRECIPE::Init, cfg
; create all filetypes (FILE<i>) from the configuration file
hcfg = cfg->Locate('FILE')
hcfg = hcfg->GetFirst()
WHILE OBJ_VALID(hcfg) DO BEGIN
    self.filetypes[self.nFiletypes] = OBJ_NEW('ldrsFILETYPE', self, hcfg)
    self.nFiletypes += 1
    hcfg = hcfg->GetNext()
ENDWHILE
; create all modles (MODULE<i>) from the configuration file
hcfg = cfg->Locate('MODULE')
hcfg = hcfg->GetFirst()
WHILE OBJ_VALID(hcfg) DO BEGIN
    self.modules[self.nModules] = OBJ_NEW('ldrsMODULE', self, hcfg)
    self.nModules += 1
    hcfg = hcfg->GetNext()
ENDWHILE
; finished
RETURN, 1
END

FUNCTION ldrsRECIPE::GetName
RETURN, self.recipeName
END

PRO ldrsRECIPE::SetName, name
self.recipeName = name
END

FUNCTION ldrsRECIPE::GetNModules
RETURN, self.nModules
END

FUNCTION ldrsRECIPE::GetModule, nr
IF nr GE self.nModules THEN RETURN, OBJ_NEW()
RETURN, self.modules[nr]
END

FUNCTION ldrsRECIPE::FindModule, tag
FOR i = 0,self.nModules - 1 DO BEGIN
    IF self.modules[i]->GetTag() EQ tag THEN RETURN, self.modules[i]
    IF self.modules[i]->GetLabel() EQ tag THEN RETURN, self.modules[i]
END
RETURN, OBJ_NEW()
END

FUNCTION ldrsRECIPE::GetNFiletypes
RETURN, self.nFiletypes
END

FUNCTION ldrsRECIPE::GetFiletype, nr
IF nr GE self.nFiletypes THEN RETURN, OBJ_NEW()
RETURN, self.filetypes[nr]
END

FUNCTION ldrsRECIPE::FindFiletype, tag
FOR i = 0, self.nFiletypes - 1 DO BEGIN
    IF self.filetypes[i]->GetTag() EQ tag THEN RETURN, self.filetypes[i]
ENDFOR
RETURN, OBJ_NEW()
END

FUNCTION ldrsRECIPE::GetNViews
RETURN, self.nViews
END

PRO ldrsRECIPE::AddView, view
;PRINT, 'adding view ', view, ' to the recipe'
IF self.nViews GE 100 THEN RETURN
self.views[self.nViews] = view
self.nViews += 1
END

FUNCTION ldrsRECIPE::GetView, nr
IF nr GE self.nViews THEN RETURN, OBJ_NEW()
RETURN, self.views[nr]
END

PRO ldrsRECIPE::Broadcast, method, obj
FOR i = 0, self.nViews - 1 DO CALL_METHOD, method, self.views[i], obj
END

PRO ldrsRECIPE::SelectModule, module
self->DeselectModule
self->DeselectFiletype
;PRINT, "SelectModule ", module->GetTag()
self.selModule = module
FOR i = 0, self.nViews - 1 DO self.views[i]->ModuleSelected, module
END

PRO ldrsRECIPE::DeselectModule
IF OBJ_VALID(self.selModule) THEN BEGIN
    ;PRINT, "DeselectModule ", self.selModule->GetTag()
    FOR i = 0, self.nViews - 1 DO self.views[i]->ModuleDeselected, self.selModule
    self.selModule = OBJ_NEW()
ENDIF
END

FUNCTION ldrsRECIPE::GetSelectedModule
RETURN, self.selModule
END

PRO ldrsRECIPE::SelectAlgorithm, algorithm
self->DeselectAlgorithm
;PRINT, "SelectAlgorithm ", algorithm->GetTag()
self.selAlgorithm = algorithm
FOR i = 0, self.nViews - 1 DO self.views[i]->AlgorithmSelected, algorithm
module = algorithm->GetModule()
module->SelectAlgorithm, algorithm
END

PRO ldrsRECIPE::DeselectAlgorithm
IF OBJ_VALID(self.selAlgorithm) THEN BEGIN
    ;PRINT, "DeselectAlgorithm ", self.selAlgorithm->GetTag()
    FOR i = 0, self.nViews - 1 DO self.views[i]->AlgorithmDeselected, self.selAlgorithm
    self.selAlgorithm = OBJ_NEW()
ENDIF
END

PRO ldrsRECIPE::SelectFiletype, filetype
self->DeselectModule
self->DeselectFiletype
;PRINT, "SelectFiletype ", filetype->GetTag()
self.selFiletype = filetype
FOR i = 0, self.nViews - 1 DO self.views[i]->FiletypeSelected, filetype
END

PRO ldrsRECIPE::DeselectFiletype
IF OBJ_VALID(self.selFiletype) THEN BEGIN
    ;PRINT, "DeselectFiletype ", self.selFiletype->GetTag()
    FOR i = 0, self.nViews - 1 DO self.views[i]->FiletypeDeselected, self.selFiletype
    self.selFiletype = OBJ_NEW()
ENDIF
END

FUNCTION ldrsRECIPE::GetSelectedFiletype
RETURN, self.selFiletype
END

PRO ldrsRECIPE::SelectFile, file
self->DeselectFile
;PRINT, "SelectFile ", file->GetName()
self.selFile = file
FOR i = 0, self.nViews - 1 DO self.views[i]->FileSelected, file
END

PRO ldrsRECIPE::DeselectFile
IF OBJ_VALID(self.selFile) THEN BEGIN
    ;PRINT, "DeselectFile ", self.selFile->GetName()
    FOR i = 0, self.nViews - 1 DO self.views[i]->FileDeselected, self.selFile
    self.selFile = OBJ_NEW()
ENDIF
END

FUNCTION ldrsRECIPE::GetSelectedFile
RETURN, self.selFile
END

PRO ldrsRECIPE::SelectImage, image
self->DeselectImage
self.selImage = image
FOR i = 0, self.nViews - 1 DO self.views[i]->ImageSelected, image
END

PRO ldrsRECIPE::DeselectImage
IF self.selImage NE -1 THEN BEGIN
    FOR i = 0, self.nViews - 1 DO self.views[i]->ImageDeselected, self.selImage
    self.selImage = -1
ENDIF
END

PRO ldrsRECIPE::ImageChanged, file, image
IF OBJ_VALID(file) THEN BEGIN
    ;PRINT, "Image Changed ", file->GetName(), image
    FOR i = 0, self.nViews - 1 DO self.views[i]->ImageChanged, file, image
ENDIF
END

PRO ldrsRECIPE::Store, cfg
child = cfg->Locate('MODULE')
FOR i=0, self.nModules - 1 DO BEGIN
    hchild = child->Locate(self.modules[i]->GetTag())
    self.modules[i]->Store, hchild
ENDFOR
child = cfg->Locate('FILE')
FOR i=0, self.nFiletypes - 1 DO BEGIN
    hchild = child->Locate(self.filetypes[i]->GetTag())
    self.filetypes[i]->Store, hchild
ENDFOR
END

PRO ldrsRECIPE::Load, cfg
self->DeselectModule
self->DeselectAlgorithm
self->DeselectFiletype
self->DeselectFile
self->DeselectImage
child = cfg->Locate('MODULE')
FOR i=0, self.nModules - 1 DO BEGIN
    hchild = child->Locate(self.modules[i]->GetTag())
    self.modules[i]->Load, hchild
ENDFOR
child = cfg->Locate('FILE')
FOR i=0, self.nFiletypes - 1 DO BEGIN
    hchild = child->Locate(self.filetypes[i]->GetTag())
    self.filetypes[i]->Load, hchild
ENDFOR
END

PRO ldrsRECIPE::Read, fileName
self->SetName, fileName
cfg = OBJ_NEW('ldrsCFG', OBJ_NEW(), '', -1)
;PRINT, '  name=', self.recipeName
OPENR, lun, fileName, ERROR=err, /GET_LUN
;PRINT, '  lun=', lun
;PRINT, '  err=', err
IF err NE 0 THEN BEGIN
    PRINTF, -2, !ERROR_STATE.MSG
    RETURN
END
cfg->Load, lun
self->Load, cfg
OBJ_DESTROY, cfg
FREE_LUN, lun
FOR i = 0, self.nViews - 1 DO self.views[i]->RecipeLoaded, self
END

PRO ldrsRECIPE::Write, fileName
self->SetName, fileName
; create a recipe as an internal configuration tree
cfg = OBJ_NEW('ldrsCFG', OBJ_NEW(), '', -1)
self->Store, cfg
;PRINT, '  name=', self.recipeName
OPENW, lun, fileName, ERROR=err, /GET_LUN
;PRINT, '  lun=', lun
;PRINT, '  err=', err
IF err NE 0 THEN BEGIN
    PRINTF, -2, !ERROR_STATE.MSG
    RETURN
END
cfg->Save, lun, ''
OBJ_DESTROY, cfg
FREE_LUN, lun
END

PRO ldrsRECIPE__define
  struct = { ldrsRECIPE, $
             recipeName:'', $
             nViews:0, $
             views:OBJARR(100), $
             nModules:0, $
             modules:OBJARR(10), $
             selModule:OBJ_NEW(), $
             selAlgorithm:OBJ_NEW(), $
             nFiletypes:0, $
             filetypes:OBJARR(32), $
             selFiletype:OBJ_NEW(), $
             selFile:OBJ_NEW(), $
             selImage:-1 $
           }
END
